//---------------------------------------------------------------------------
// WAP.c
// Contains all platform independant WAP code 
//
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
//
// $Workfile: WAP.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.8 $
//
// $Header: /cvsroot/wapuniverse/wapuniverse/src/WAP.c,v 1.8 2000/11/11 20:51:09 wapuniverse Exp $
//

//---------------------------------------------------------------------------

#include "build.h"
#include "WAPUniverse.h"
#include "wsp.h"
#ifdef __palmos__
    #include "dbConn.h"
    #include "dbUrl.h"
#endif



int StringToIPAddr (char *str, unsigned long *addr)
{
//    struct hostent *hp = NULL;
#ifdef __palmos__
    NetHostInfoBufType AppHostInfo;     // DO NOT DELETE !!
	// without this AppHostInfo, gethostbyname() corrupts memory!!!!!!!
#endif

// WAP Gateway can only be IP, no DNS name
//    if ((hp = gethostbyname (str)) == NULL) {
    if ((long)(*addr = inet_addr (str)) == -1)
        return 0;
//    }
//    else
//      bcopy ((char *)(hp->h_addr), (char *)addr, hp->h_length);

    return 1;
}

char * 
gstrcopy(char *dst, const char *src)
{
    char *tmp=dst;

    if ( dst==NULL || src==NULL)
        return NULL;
    while (*src)
        *tmp++ = *src++;
    *tmp=0; 
    return(dst);
}


int 
gstrlen(char *str)
{
    int i=0;

    if ( str == NULL)
        return(0);

    while (*str++)
        i++;
    return(i);
}


void
enc_uintvar(char buf[], int *i, int intval)
{
    int j,p;

    for (j=4; j>1 ; j--) {
        p = intval>>(7*j) ;
        if (p> 0) {
            buf[*i] = (p);
            (*i)++;
        }
    }
    buf[*i]= intval;
    (*i)++;

}


// connect2server: open the connection to the 'hostname' server
// and return the socket descriptor
int Connect2Server(char *gwip, char *urlStr, GlobalsType *g )
{
    unsigned long host;
    int sock;

    if (StringToIPAddr(gwip,&host) != 1) {
        DisplayError("Invalid WAP Gateway IP");
        return -1;
    }

    memset((char *) &(g->RmtAddress), 0, sizeof(g->RmtAddress));
    g->RmtAddress.sin_family = AF_INET;
    g->RmtAddress.sin_port = htons(WAP_WSP_CL);
    g->RmtAddress.sin_addr.s_addr = host;

    sock = socket(AF_INET, SOCK_DGRAM, 0);             

    g->LclAddress.sin_family = AF_INET;
    g->LclAddress.sin_port   = htons(WAP_WSP_CL_LOCAL);
    g->LclAddress.sin_addr.s_addr = INADDR_ANY;

    if (bind(sock, (struct sockaddr *) &g->LclAddress, sizeof(g->LclAddress)) < 0) {
        MyErrorFunc("bind failed",WspHostErrStr());
        return -2;
    }

    return(sock);
}

int wsp_get(int sock, char *gwip, char *urlStr, Boolean remember, GlobalsType *g )
// Boolean remember: true if we are loading a WML page (and not an image), sowe set gLastURL to current URL, needed for relative links later on
{
    unsigned char outbuf[256];
    int i = 0;

    // See WSP.PDF for more info
    outbuf[i++]=0x01; // Transaction ID
    outbuf[i++]=WSP_PDU_GET; 
    enc_uintvar(outbuf,&i,gstrlen(urlStr)); // URIlen , uintvar !!
    gstrcopy((outbuf+i), urlStr);
	if(remember)
		gstrcopy(g->WapLastUrl,urlStr);
    i+= gstrlen(urlStr);
    // Start of headers

    outbuf[i++]=0x81; // 01 Accept Charset
    outbuf[i++]=0xEA; // 6A = UTF-8
    outbuf[i++]=0x81; // 01 Accept Charset
    outbuf[i++]=0x84; // 04 iso-8859-1 
    outbuf[i++]=0x81; // 01 Accept Charset
    outbuf[i++]=0x03; // 03 us-ascii
    outbuf[i++]=0x02; // 02 Accept Encoding
    outbuf[i++]=0x03; // 03 Token Text ???????
    outbuf[i++]=0xE8; // 68
    outbuf[i++]=0x80; // 00 Accept
    outbuf[i++]=0x94; // 14 application/vnd.wap.wmlc
    outbuf[i++]=0x80; // 00 Accept
    outbuf[i++]=0x95; // 15 application/vnd.wap.wmlscriptc
    outbuf[i++]=0x80; // 00 Accept
    outbuf[i++]=0xA1; // 21 image/vnd.wap.wbmp
    outbuf[i++]=0xA9; // 29 User Agent
    gstrcopy(&(outbuf[i]),AGENT_STRING);
    i+= gstrlen(AGENT_STRING);
    outbuf[i++]=0x00;
    if (sendto(sock,&outbuf,i,0, (struct sockaddr *) &g->RmtAddress,sizeof(g->RmtAddress))<0)
        MyErrorFunc("sendto failed", WspHostErrStr());

    return sock;
}


int wapCreateUrlStr(char target[], char url[], GlobalsType *g)
{
int j,i = 0;

  if (!target)
        return(-1);  	  
  if (target[4] == ':'){ // looks like we got 'http://'
	  gstrcopy(url,target);
      return(0);
  }
  if (target[0] == '/'){ // looks like we have relative to server
	  // yep, copy 'http://'
	  while ( (g->WapLastUrl[i]!= 0) && (i<7)){
    	  url[i]=g->WapLastUrl[i++];
	  }
  	  // We need till next '/'
	  while ( (g->WapLastUrl[i]!= 0) && (g->WapLastUrl[i]!= '/')){
    	  url[i]=g->WapLastUrl[i++];
	  }
	  gstrcopy(&url[i],target);
	  return(0);
  }else{  // well ... relative to last URL
	  while (g->WapLastUrl[i]!= 0){
	  	  if (g->WapLastUrl[i] == '/'){
		  		j = i;
		  }
    	  url[i]=g->WapLastUrl[i++];
	  }
	  gstrcopy(&url[j+1],target);
	  return(0);
      
  }	  
}

// returns socket
int wapGetUrl(dbConnConnection *conn, char *urlstr, Boolean remember, GlobalsType *g )
// Boolean remember: true if we are loading a WML page (and not an image), so we set gLastURL to current URL, needed for relative links later on
{
    int         sock=0;

    sock = Connect2Server(conn->ipaddress,urlstr,g);       
    if (sock < 0) {
        return(-1);
    }
    wsp_get(sock,conn->ipaddress,urlstr,remember,g);


    return(sock);
}

