// ---------------------------------------------------------------------------
// WAPUniverse.c
// Contains the main PalmOS specific code, including PilotMain
// 
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
// 
// $Workfile: WAPUniverse.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.11 $
// 
// $Header: /cvsroot/wapuniverse/wapuniverse/src/WAPUniverse.c,v 1.11 2000/11/11 20:51:09 wapuniverse Exp $
// 
// -------------------------------------------------------------------------------


#include    <PalmOS.h>
#include    <SysEvtMgr.h>
#include 	"WAPUniverse.h"
#include	"build.h"
#include 	"../res/WAPUniverse_res.h"
#include    "formUrlList.h"
#include    "formConnectionList.h"
#include    "formEditConnection.h"
#include    "formEditUrl.h"
#include    "formBrowser.h"
#include    "frmScroll.h"
#include	"formMTSyncPrefs.h"

static UInt16 RomVersionCompatible (UInt32 requiredVersion, UInt16 launchFlags);
static Boolean ApplicationHandleEvent ( EventPtr event );
static int StartApplication(void);
static void EventLoop(void);
static void StopApplication(void);

// Following globals are needed if you do not like crashes in the network lib
Int32            AppNetTimeout;
UInt16           AppNetRefnum; // used by SysLibFind
Err 			 errno; 	   // Berkely socketinterface 

// globals for internal use
//DmOpenRef    		gDbUrlDb;          // handle for URL database (bookmarks)
//DmOpenRef    		gDbConnDb;          // handle for Connection database 
//UInt16       		gCurrentView;    // id of current form


UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	int error;

    error = RomVersionCompatible (version20, launchFlags);
    if (error)
       return error;
	   
	if (cmd == sysAppLaunchCmdNormalLaunch)
 	{
		error = StartApplication();	// Application start code
		if (error) return error;
		EventLoop();			// Event loop
		StopApplication ();		// Application stop code
	}
	return 0;
}		

gWAPAppInfoType gWAPAppInfo =
{
  // Renamed categories
  0x0008,                   // set the 2nd and 3rd bits indicating those
                                // categories as changed.
  {
    "Unfiled",
    "Business",
    "Personal",
    "History",
    "", "", "", "",
    "", "", "", "",
    "", "", "", ""
  },
    // Category Uniq IDs
  {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
    // Last Uniq ID
  15,
    // Reserved
  0, 0
};


void SwitchForm(UInt16 formID, GlobalsType *g)
{
	FrmGotoForm(formID);
	g->CurrentView = formID;
}

void DisplayLicense()
{
MemHandle licenseStr;
		
	licenseStr = DmGetResource (strRsc, strGPL);
	ScrollFrm ("WAPUniverse License", licenseStr);
	
}

void DisplayAbout()
{
          ControlPtr      ctl;
          FormPtr         frm;
          
          MenuEraseStatus (0);
          // Load the info form, then display it.
          frm = FrmInitForm (frmInfo);
          ctl = (ControlPtr)(FrmGetObjectPtr(frm,(FrmGetObjectIndex(frm,lblVersion))));
          CtlSetLabel(ctl,VERSIONSTR);
          ctl = (ControlPtr)(FrmGetObjectPtr(frm,(FrmGetObjectIndex(frm,lblBuildDate))));
          CtlSetLabel(ctl,BUILDSTR	);
          FrmDoDialog (frm);
          // Delete the info form. 
          FrmDeleteForm (frm);
}


static int StartApplication(void)
{
  UInt16     	mode;         // permissions when opening the database
  GlobalsType 	*g;		      // all the globals

  	// Check if secret record should be shown.
  	if (PrefGetPreference (prefHidePrivateRecordsV33))
    	mode = dmModeReadWrite;
  	else
    	mode = dmModeReadWrite | dmModeShowSecret;

	g = (GlobalsType *)MemPtrNew(sizeof(GlobalsType));
 	MemSet(g, sizeof(GlobalsType), 0);
 	FtrSet(wuAppType, ftrGlobals, (UInt32)g);
	g->state = BS_IDLE;
	g->CurrentCategory = dmAllCategories;
	g->Render = true;
    dbConnOpenOrCreateDB(mode,g);
    dbUrlOpenOrCreateDB(mode,g);
    if (dbConnNumRecords(g) >0)
	   g->CurrentView = frmUrlList;
	else
	   g->CurrentView = frmConnectionList;
    FrmGotoForm (g->CurrentView);
	return 0;
}

static Boolean ApplicationHandleEvent(EventPtr event)
{
    FormPtr frm;
    Int16   formId;
    Boolean handled = false;

    if (event->eType == frmLoadEvent) {
        formId = event->data.frmLoad.formID;
        frm = FrmInitForm(formId);
        FrmSetActiveForm(frm);
        
        switch(formId) {

         case frmUrlList:
             FrmSetEventHandler(frm, ehFormUrlList);
             break;
         case frmEditURL:
             FrmSetEventHandler(frm, ehFormEditURL);
             break;
         case frmBrowser:
             FrmSetEventHandler(frm, ehFormBrowser);
             break;
         case frmEditConnection:
             FrmSetEventHandler(frm, ehFormEditConnection);
             break;
         case frmConnectionList:
             FrmSetEventHandler(frm, ehFormConnectionList);
             break;
// 

         default:
	     break;
            }
        handled = true;
        }
    return handled;
}

void EventLoop(void)
{
    EventType event;
    UInt16 error;

    do {
        EvtGetEvent(&event, sysTicksPerSecond/5);  
        if (! SysHandleEvent(&event))
            if (! MenuHandleEvent(0, &event, &error))
                if (! ApplicationHandleEvent(&event))
                   FrmDispatchEvent(&event);
        } while (event.eType != appStopEvent);
}

static void StopApplication(void)        
{
  GlobalsType *g;

  // get globals reference
  FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);
  FrmCloseAllForms ();
  DmCloseDatabase (g->DbUrlDb);
  DmCloseDatabase (g->DbConnDb);
  MemPtrFree(g);
}


static UInt16 RomVersionCompatible (UInt32 requiredVersion, UInt16 launchFlags)
{
  UInt32           romVersion;
  
  FtrGet (sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if (romVersion < requiredVersion)
    {
      if ((launchFlags & (sysAppLaunchFlagNewGlobals |
          sysAppLaunchFlagUIApp)) ==
          (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
          FrmAlert (alRomIncompatible);
          if (romVersion < 0x02000000){
            AppLaunchWithCommand (sysFileCDefaultApp,
                                  sysAppLaunchCmdNormalLaunch, NULL);
	  }
        }
      return (sysErrRomIncompatible);
    }
  return 0;
}

