#ifndef _wapuniverse_h_
#define _wapuniverse_h_
//---------------------------------------------------------------------------
// MobileToken 
// Contains all PalmOS User Interface specific code, including PilotMain
//
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
//
// $Workfile: MobileToken.h $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.10 $
//
// $Header: /cvsroot/wapuniverse/wapuniverse/src/WAPUniverse.h,v 1.10 2000/11/11 20:51:09 wapuniverse Exp $
//
//---------------------------------------------------------------------------
#include <PalmOS.h>
#include <SysEvtMgr.h>
#include <sys_socket.h>

#define SEC_1  __attribute__ ((section ("sec_1")))

#define version20 			0x02000000  // PalmOS 2.0 version number
#define wuAppType     		'gWAP'      // type for application.  must be 4 chars, mixed case.
#define noRecordSelected    -1          // index for no current record.
#define localizedAppInfoStr 1000    // default category strings ** tAIS **
#define REPLY_TIMEOUT    	20 // 20 seconds maximum timeout
#define MAX_URL_LEN 		255

// states for FormBrowser Statemachine
#define BS_IDLE         1
#define BS_GET_URL      2
#define BS_GET_REPLY    3
#define BS_WSP_DECODE   4
#define BS_WBXML_PARSE  5
#define BS_RENDER		6
#define BS_NEW_CARD     7
#define BS_ONTIMER		8
#define BS_GET_OFFLINE  9

extern void MyErrorFunc (char* error, char* additional);
#define DisplayError(str)	MyErrorFunc(str,__FILE__);  

#define ftrGlobals         1000

// history.c
// history: the stack containing the URL history
#define MAX_HISTORY	  20 // min. 10 cfr SPEC-WML 12.2.1 

typedef struct historyEntryType {
  Char	*url;
  // TODO : add cache location and exporation here as well
} historyEntryType;

typedef struct historyType {
  historyEntryType	hist[MAX_HISTORY]; 
  int				index;
} historyType;

typedef struct historyType *historyPtr;

// WML.C
// CardType: used to  build an internal list of all cards in a deck
typedef struct CardType {
  Char				*card_id;
  int 				offset;
  struct CardType	*next; 
} CardType;

typedef struct CardType *CardPtr;


typedef struct WMLDeckStruct *WMLDeckPtr;

typedef struct WMLDeckStruct {
  int 		    version;
  int 		    publicID;
  int 		    charSet;
  char   	    *strTable;
  int		    decksize;
  int		    offset;
  unsigned char tag;
  int		    action;
  int		    stackDepth;
  unsigned char *stack;
  int		    maxstack;
  int		    template;
  CardPtr		card;
  int			timerVal;   // for ontimer event, -1 if no timer
  unsigned char *ontimerUrl;  
  unsigned char	*data;
  // temp vars, put here to avoid stack overflows.
  unsigned char *attrstr;
  unsigned char *tmpstr;
} WMLDeck ;

// dbURL.c
// Connection Settings record	
typedef struct {
    char    name[31];
    char    urlstr[MAX_URL_LEN+1];
    char    connection[16];
} dbUrlURL;

typedef dbUrlURL * dbUrlURLPtr;

typedef struct {
    char    name[1];    // actually may be longer than 1
} dbUrlPackedURL;



// Connection Settings record, mostly used in dbConn.c
typedef struct {
    char    name[31];
    char    ipaddress[16];      // e.g. 172.116.212.123 
    char    security;           // 'Y'es or 'N'o
    char    bearer;             // 'C'SD or 'S'MS
    char    connectionType;     // C'O' or C'L'
} dbConnConnection;

typedef dbConnConnection * dbConnConnectionPtr;

typedef struct {
    char    name[1];    // actually may be longer than 1
} dbConnPackedConnection;

  
typedef struct {
    UInt16   renamedCategories;        // bitfield of categories with a changed name
    Char     categoryLabels[dmRecNumCategories][dmCategoryLength];
    UInt8    categoryUniqIDs[dmRecNumCategories];
    UInt8    lastUniqID;         // Uniq IDs for categories generated by the device 
    // are between 0 - 127.  Those from the PC are 128 - 255.
    UInt8    reserved1;          // from the compiler UInt16 aligning things
    UInt16   reserved2;
} gWAPAppInfoType;

typedef gWAPAppInfoType * gWAPAppInfoPtr; 

typedef struct HyperLinkType {
  Int16 x1;
  Int16 y1;
  Int16 x2;
  Int16 y2;
  Char *url;
  struct HyperLinkType *next;
} HyperLinkType;

typedef HyperLinkType *HyperLinkPtr;

typedef struct VarType {
  Char *name;
  Char *value;
  struct VarType *next;
} VarType;

typedef VarType *VarPtr	;

typedef struct InputType {
  Int16 x1;
  Int16 y1;
  Int16 x2;
  Int16 y2;
  Char *name;
  Boolean passwordType;  // true = Password/ false = text
  Char *format;
  Boolean emptyOK;
  Int16 size;
  Int16 maxlength;
  struct InputType *next;
} InputType;

typedef InputType *InputPtr;

typedef struct OptionType {
  Char 				*value;
  Char 				*txt;
  struct OptionType *next;
} OptionType;

typedef OptionType *OptionPtr;

typedef struct SelectType {
  Int16 	x1;
  Int16 	y1;
  Int16 	x2;
  Int16 	y2;
  Char 		*name;
  OptionPtr options;
  struct SelectType *next;
} SelectType;

typedef SelectType *SelectPtr;

// 

// WSP.C
typedef struct {
  unsigned char transaction_id;
  unsigned char pdu_type;
  unsigned char status_code;
  unsigned char *data;
  int length;
} WSPPDU ;

typedef WSPPDU * WSPPDUPtr;

typedef struct {
	InputPtr        	InputFldActive;
	Boolean      		NetLibOpen;
	WSPPDU       		wsppdu;
	MemHandle    		srcHandle;
	Int16        		x,y;
	Int16        		state;
	char         		*inBuf;
	Int16        		sock;
	dbConnConnection	conn;
	MemHandle    		resBmpPrgrs[4];
	Int16        		progress;
	WinHandle    		DrawWin;
	Int16        		DrawHeight;
	Int16   	     	ScrollY;
	HyperLinkPtr     	HyperLinks; // linked list of hyperlinks
	InputPtr			Input; 		// linked list of input elements
	VarPtr				Var; 		// linked list of variables
	SelectPtr			Select;
	Char				*Url;
	WMLDeck         	wmldck;
	Int16				CardOffset;
	DmOpenRef    		DbUrlDb;           // handle for URL database (bookmarks)
	DmOpenRef    		DbConnDb;          // handle for Connection database 
	UInt16       		CurrentView;       // id of current form
	gWAPAppInfoType 	WAPAppInfo;
	dbConnConnection 	CurrentConnection;
	Char *        		EditURLConnection;
	MemHandle       	ConnectionsHandle;
	MemHandle       	ConnectionsListArrayH;    // for the connectionslist in the URLEditForm
	historyPtr 			History;
	char 				WapLastUrl[MAX_URL_LEN+1];
	Boolean 			NewCategoryEdited;	// true if category name has been edited
	Int16      			DbConnCurrentRecord;       // index of current Connection database record
	Int16        		DbUrlCurrentRecord;       // index of current URL database record
	dbUrlURL     		DbUrlCurrentURL;
	Int16 		 		DbUrlCurrentCategory;
	char         		DbUrlCategoryName[dmCategoryLength];     // Storage for the name
	char           		ActiveConnection[16];
	Int16            	CurrentCategory;  // category filter
	UInt16           	RecordCategory;   // edited record's category (never All)
	UInt16           	NewCategory;    // category in details form (never All)
	Boolean 			Render;
	struct sockaddr_in 	RmtAddress;
	struct sockaddr_in 	LclAddress;
} GlobalsType;

/* 

 // get globals reference
GlobalsType *g;
 FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);


 // create the globals object, and register it
GlobalsType *g;
 globals = (Globals *)MemPtrNew(sizeof(Globals));
 MemSet(globals, sizeof(Globals), 0);
 FtrSet(wuAppType, ftrGlobals, (UInt32)globals);

*/

void SwitchForm(UInt16 formID, GlobalsType *g);
void DisplayAbout();
void DisplayLicense();

#endif

