// ---------------------------------------------------------------------------
// FormBrowser.c
// Contains PalmOS specific code, for Browser form
// 
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
// 
// $Workfile: formBrowser.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.21 $
// 
// $Header: /cvsroot/wapuniverse/wapuniverse/src/formBrowser.c,v 1.21 2000/11/11 20:51:09 wapuniverse Exp $
// 
// -------------------------------------------------------------------------------
#include    <PalmOS.h>
#include    <SysEvtMgr.h>
#include 	"WAPUniverse.h"
#include    "PalmUtils.h"
#include	"build.h"
#include 	"../res/WAPUniverse_res.h"
#include    "formUrlList.h"
#include    "formConnectionList.h"
#include    "formEditConnection.h"
#include    "formEditUrl.h"
#include    "dbConn.h"
#include    "frmScroll.h"
#include 	<NetMgr.h>
#include 	"wml.h"
#include    "history.h"
#include    "formBrowser.h"
#include    "MenuHnd.h"
#include 	"dbOffline.h"
#ifdef __GNUC__
#include "callbacks.h"
#endif

#define LINEHEIGHT      12


static Boolean browserPenUpEvt(EventPtr event, GlobalsType *g);
void palmDrawChars(char *str, Int16 textLen, Int16 x, Int16 y, GlobalsType *g);
Boolean selectInitList (SelectPtr slct, GlobalsType *g);
static void scrollUpdate(GlobalsType *g);


void MyErrorFunc (char* error, char* additional)
{

    FrmCustomAlert (alError, error, additional ? additional : "", NULL);
}

static void updateScreen(GlobalsType *g)
{
    WinHandle   tmpDrawWin;
    RectangleType rect={{0, 0}, {153, 132}};

    tmpDrawWin = WinGetDrawWindow();
    WinCopyRectangle(g->DrawWin,tmpDrawWin,&rect,0,LINEHEIGHT,winPaint);
}

void setVar(Char *name, Char *value, GlobalsType *g)
{
  VarPtr tmp;
  
  	tmp= g->Var;
	while(tmp != NULL)
	{
		
		if (StrCompare(name,tmp->name)==0)
		{
			if(tmp->value)
				Free(tmp->value);
			tmp->value = Malloc(StrLen(value)+1);
			ErrFatalDisplayIf (!tmp->value, "Malloc Failed");
			StrCopy(tmp->value,value);				
			return;			
		}
		tmp = tmp->next;
	}
}


Boolean selectInitList (SelectPtr slct, GlobalsType *g)
{
  ListPtr         optionslstPtr;
  UInt16          nrOptions;
  UInt16		  offset=0;
  FormPtr         frm;
  Char            *txt;
  Char 			  *options;
  Err             error;
  MemHandle		  optionsHandle,optionsListArrayH;		  
  OptionPtr       tmp;
  WinHandle   	  prevDrawWin;
  RectangleType   rect;
  
  nrOptions = 0;
  optionsHandle = MemHandleNew (2);
  options = MemHandleLock (optionsHandle);
  *options = 0;
  tmp = slct->options;
  while(tmp != NULL)
  { 
	  if(tmp->txt != NULL){
	      MemHandleUnlock (optionsHandle);
	      error = MemHandleResize (optionsHandle,StrLen(tmp->txt)+
                               offset + 2);
	      options = MemHandleLock (optionsHandle);
	      StrCopy(options+offset,tmp->txt);
	      offset += (StrLen(tmp->txt)+1);
      	  nrOptions++;
	  }
	  tmp=tmp->next;
  }
  *(options+offset)=0;
  if (nrOptions == 0)
    {
      MemHandleUnlock (optionsHandle);
	  MemHandleFree (optionsHandle);
      return(false);
    }
  optionsListArrayH = SysFormPointerArrayToStrings (options, nrOptions);
  frm = FrmGetActiveForm();
  optionslstPtr = (ListPtr) (FrmGetObjectPtr (frm, (FrmGetObjectIndex
                                                        (frm, listOption))));
  LstSetListChoices (optionslstPtr, MemHandleLock
                     (optionsListArrayH), nrOptions);
  LstSetHeight (optionslstPtr, nrOptions);
//  LstSetSelection(optionslstPtr,0); // current connection 
  LstPopupList(optionslstPtr);
  txt = LstGetSelectionText(optionslstPtr,
                          LstGetSelection(optionslstPtr));
  if (txt){
	prevDrawWin = WinGetDrawWindow();
 	WinSetDrawWindow(g->DrawWin);
	rect.topLeft.x = slct->x1+16;
	rect.extent.x = 144-slct->x1 ;
	rect.topLeft.y = slct->y1-LINEHEIGHT;
	rect.extent.y = LINEHEIGHT ;
	WinEraseRectangle(&rect,0);
	WinDrawChars(txt, StrLen(txt), slct->x1+18, slct->y1-LINEHEIGHT);
	WinSetDrawWindow(prevDrawWin);
	scrollUpdate(g);
 	if(slct->name){
		//value = getVar(slct->name);
		tmp = slct->options;
		while(tmp != NULL)
		{
			if (StrCompare(txt,tmp->txt)==0)
				setVar(slct->name,tmp->value,g);
			tmp=tmp->next;
		}
	}
  }
  MemHandleUnlock (optionsHandle);
  MemHandleFree (optionsHandle);
  return(true);   
}

void deactivateInputFld(GlobalsType *g){
  FieldPtr      fldP;
  FormPtr       frm;
  Char 			*txt;
  InputPtr		tmp;
      
	if(g->InputFldActive) // if there is an active input field, dactivate it 
	{				
		frm = FrmGetActiveForm ();
	  	fldP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,fldInput));
  		ErrNonFatalDisplayIf (!fldP, "missing field");
  		txt = FldGetTextPtr (fldP);
		tmp = g->InputFldActive;
	    setVar(tmp->name,txt,g);
		g->InputFldActive=NULL;
		FrmHideObject(FrmGetActiveForm(),FrmGetObjectIndex(frm,fldInput));
		palmDrawChars(txt, StrLen(txt), tmp->x1, tmp->y1-(g->ScrollY)-LINEHEIGHT,g);
	}				

}

void setHyperLinkUrl(Char *prefix, Char *url, GlobalsType *g)
{
	ErrFatalDisplayIf (!g->HyperLinks, "!gHyperLinks");
	ErrFatalDisplayIf (!url, "No URL");
	g->HyperLinks->url = Malloc(StrLen(url)+StrLen(prefix)+1);
	ErrFatalDisplayIf (!g->HyperLinks->url, "Malloc Failed");
	StrCopy(g->HyperLinks->url,prefix);
	StrCopy(g->HyperLinks->url+StrLen(prefix),url);
}

int addHyperLink(Int16 x1, Int16 y1, GlobalsType *g)
{
  HyperLinkPtr tmp;

	tmp = Malloc (sizeof(HyperLinkType));
	ErrFatalDisplayIf (!tmp, "Malloc Failed");
    tmp->x1=x1;
    tmp->y1=y1;
	tmp->next = g->HyperLinks;
	tmp->url=NULL;
    g->HyperLinks = tmp;
			
return(0);
}

int addVar(Char *name, GlobalsType *g)
{
  VarPtr tmp;

	tmp = Malloc (sizeof(VarType));
	ErrFatalDisplayIf (!tmp, "Malloc Failed");
	tmp->value=NULL;
	tmp->name = Malloc(StrLen(name)+1);
	ErrFatalDisplayIf (!tmp->name, "Malloc Failed");
	StrCopy(tmp->name,name);
	tmp->next = g->Var;
    g->Var = tmp;
			
return(0);
}

Char *getVar(Char *name, GlobalsType *g)
{
  VarPtr tmp;
  
  	tmp= g->Var;
	while(tmp != NULL)
	{
		
		if (StrCompare(name,tmp->name)==0)
		{
			if(tmp->value)
				return(tmp->value);
			else
				return("");
		}
		tmp = tmp->next;
	}
    return("");
}

void setHyperLinkEnd(Int16 x2, Int16 y2, GlobalsType *g)
{
 if(!g->HyperLinks)
 	return;
 g->HyperLinks->x2 = x2;
 g->HyperLinks->y2 = y2; 
}

char *escaped(Char *str)
{
 // TODO : escaped string
 return(str);
}

char *unescaped(Char *str)
{
 // TODO : unescaped string
 return(str);
}

unsigned char *expandVar(unsigned char *instr, GlobalsType *g)
{
int i,start,exttype,newlen,len;
unsigned char *name,*value,*str,*newstr,c;

  len = StrLen(instr);
  str = Malloc(len+1);	   
  ErrFatalDisplayIf (!newstr, "Malloc expandVar");
  StrCopy(str,instr);
  i = 0;
  while(i<len)
  {
    c = str[i];
  	if ((c==EXT_T_0) || (c==EXT_T_1) || (c==EXT_T_2)) 
	{
	  exttype= c;
	  start = i+1;
	}
	if (c==END)
	{
	   name=&(str[start]);
	   str[i]=0;
	   value = getVar(name,g);
	   if(exttype==EXT_T_0)
	     value = escaped(value);
	   if(exttype==EXT_T_1)
	     value = unescaped(value);	  
	   newlen = len - (StrLen(name)+2) + StrLen(value) + 1;
	   newstr = Malloc(newlen);	   
	   ErrFatalDisplayIf (!newstr, "Malloc expandVar");
	   StrNCopy(newstr,str,start-1);
	   StrNCopy(newstr+start-1,value,StrLen(value));
	   StrNCopy(newstr+start-1+StrLen(value),&(str[i+1]),len-i);
	   newstr[newlen-1]=0;	
       Free(str);
	   str = newstr;
  	   len = StrLen(str);
	   i=0;
	}else{
	   i++;
	}
  }
  return(str);
}

void followLink(unsigned char *str, GlobalsType *g)
{
  CardPtr      tmpCrd;
  Char *urlStr;
 
  urlStr = expandVar(str,g);
  if (urlStr[0]=='#'){
	g->state = BS_NEW_CARD;
	tmpCrd = g->wmldck.card;
	while(tmpCrd){
		if(StrCompare(tmpCrd->card_id, &urlStr[1])==0){
			g->CardOffset = tmpCrd->offset;
			Free(urlStr);
			return;
		}
		tmpCrd = tmpCrd->next;
    }
  	DisplayError("Card Not Found!");
	Free(urlStr);
	return;
  }else{
	if(g->Url)
		Free(g->Url);
    g->Url=Malloc(MAX_URL_LEN+1);
	ErrFatalDisplayIf (!g->Url, "Malloc gUrl");
    wapCreateUrlStr(urlStr,g->Url,g);
	historyPush(g->Url,g);
	Free(urlStr);
	browserReloadEvt(g);
	return;
  }  
}


Boolean checkHyperLink(Int16 x, Int16 y, GlobalsType *g)
{
  HyperLinkPtr tmp;
  
  	tmp= g->HyperLinks;
	while(tmp != NULL)
	{
		if ((y>tmp->y1)&&(y<tmp->y2)&&
				(x>tmp->x1)&&(x<tmp->x2))
		{
			deactivateInputFld(g);
			followLink(tmp->url,g);
			return(true);			
		}
		tmp = tmp->next;
	}
	return(false);
}

Boolean checkInput(Int16 x, Int16 y, GlobalsType *g)
{
  InputPtr tmp;
  ControlPtr ctl;
  RectangleType rect;
	  
  	tmp= g->Input;
	while(tmp != NULL)
	{
		if ((y>tmp->y1)&&(y<tmp->y2)&&
				(x>tmp->x1)&&(x<tmp->x2))
		{
			if (g->InputFldActive == tmp)
				return(true);
			if (g->InputFldActive)
				deactivateInputFld(g);				
 			rect.topLeft.x = tmp->x1;
			rect.extent.x = 56 ;
			rect.topLeft.y = tmp->y1-(g->ScrollY);
			rect.extent.y = 14 ;
  			ctl = GetObjectPtr (fldInput);
			FldSetBounds((FieldPtr)ctl,&rect);
			g->InputFldActive=tmp;
			SetFieldFromStr(getVar(tmp->name,g) , fldInput);
  		    FrmShowObject(FrmGetActiveForm(),FrmGetObjectIndex(FrmGetActiveForm(),fldInput));
			return(true);			
		}
		tmp = tmp->next;
	}
	return(false);
}
 
int disposeHyperLinks(GlobalsType *g)
{
  HyperLinkPtr tmp;

	while(g->HyperLinks != NULL){
		tmp = g->HyperLinks;
		g->HyperLinks = g->HyperLinks->next;
		if(tmp->url)
			Free(tmp->url);
		Free(tmp);
	}		
  	return(0);
}

int disposeVar(GlobalsType *g)
{
  VarPtr tmp;

	while(g->Var != NULL){
		tmp = g->Var;
		g->Var = g->Var->next;
		if(tmp->name)
			Free(tmp->name);
		if(tmp->value)
			Free(tmp->value);
		Free(tmp);
	}		
  	return(0);
}

int disposeOptions(OptionPtr options, GlobalsType *g)
{
  OptionPtr tmp;

	while(options != NULL){
		tmp = options;
		options = options->next;
		if(tmp->value)
			Free(tmp->value);
		if(tmp->txt)
			Free(tmp->txt);
		Free(tmp);
	}		
  	return(0);
}
 
int disposeSelect(GlobalsType *g)
{
  SelectPtr tmp;

	while(g->Select != NULL){
		tmp = g->Select;
		g->Select = g->Select->next;
		if(tmp->name)
			Free(tmp->name);
		disposeOptions(tmp->options,g);
		Free(tmp);
	}		
  	return(0);
}


Boolean checkSelect(Int16 x, Int16 y, GlobalsType *g)
{
  SelectPtr tmp;
  ControlPtr ctl;
	  
  	tmp= g->Select;
	while(tmp != NULL)
	{
		if ((y>tmp->y1)&&(y<tmp->y2)&&
				(x>tmp->x1)&&(x<tmp->x2))
		{
			deactivateInputFld(g);
  			ctl = GetObjectPtr (listOption);
			LstSetPosition((ListPtr)ctl,tmp->x1,tmp->y1-g->ScrollY);
			return(selectInitList(tmp,g));
		}
		tmp = tmp->next;
	}
	return(false);
}
 
 
int disposeInput(GlobalsType *g)
{
  InputPtr tmp;

	while(g->Input != NULL){
		tmp = g->Input;
		g->Input = g->Input->next;
		if(tmp->name)
			Free(tmp->name);
		Free(tmp);
	}		
  	return(0);
}
 
void palmprintln(GlobalsType *g)
{
    g->y+=LINEHEIGHT;
    g->x=0;
}

static void scrollUpdate( GlobalsType *g)
{
    WinHandle   tmpDrawWin;
    RectangleType rect={{0, 0}, {153, 132}};

    rect.topLeft.y = g->ScrollY;
    tmpDrawWin = WinGetDrawWindow();
    WinCopyRectangle(g->DrawWin,tmpDrawWin,&rect,0,LINEHEIGHT,winPaint);
    SclSetScrollBar(GetObjectPtr(scrollBarBrowser),(g->ScrollY/10),0,(g->DrawHeight/10)-13,13);

}

static void scrollUp(GlobalsType *g)
{

    g->ScrollY-=120;
    if (g->ScrollY < 0)
        g->ScrollY = 0;

    scrollUpdate(g);
}

static void scrollDown(GlobalsType *g)
{

    g->ScrollY+=120;
    if (g->ScrollY > (g->DrawHeight-132))
        g->ScrollY = (g->DrawHeight-132);

    scrollUpdate(g);
}

static Int16 WrapPos(char *str, Int16 width)
{
    Int16 i,wp;
    char c;
		
    i=0;
    wp=-1;
    while ( (c=str[i++]) != 0) {
        if (i == width ) {
            return(wp);
        }
        if (c==' ') {
            wp=i;
        }
    }
    return(wp);

}

void palmResizeWindow(Int16 space, GlobalsType *g)
{
    WinHandle   	tmpDrawWin;
    RectangleType 	rect={{0, 0}, {153, 0}};
	Err         	error;
	WinHandle		prevDrawWin;

    prevDrawWin = WinGetDrawWindow();
    if ( (g->y) > (g->DrawHeight-space)) { // resize offscreen window
        rect.extent.y = g->DrawHeight;
        g->DrawHeight+=50;
        tmpDrawWin = WinCreateOffscreenWindow(153,g->DrawHeight,screenFormat,&error);
        // copy window contents 
        WinSetDrawWindow(tmpDrawWin);
        WinEraseWindow();
        WinSetDrawWindow(prevDrawWin);
        WinCopyRectangle(g->DrawWin,tmpDrawWin,&rect,0,0,winPaint);
        WinDeleteWindow(g->DrawWin,false);
        g->DrawWin = tmpDrawWin;
        SclSetScrollBar(GetObjectPtr(scrollBarBrowser),0,0,(g->DrawHeight/10)-13,13);
    }
}

void palmDrawImage(WSPPDU *wsppdu, GlobalsType *g)
{
 Int16			xpos,ypos;
 UInt16 		width, height;
 UInt16 		rowBytes,wbmpRowBytes;
 BitmapType 	*bmp= NULL;
 unsigned char  *tmpPtr= NULL;
 WinHandle   	prevDrawWin;

	width = wsppdu->data[2];
	height = wsppdu->data[3];
	rowBytes =  (((width+15) >> 4) << 1); // palm bmp even word boundary !!
	wbmpRowBytes = ((width+7) >> 3) ; // wbmp is byte boundary
	bmp = MemPtrNew (sizeof(BitmapType)+(rowBytes*height));	
	ErrFatalDisplayIf (!bmp, "Malloc Failed");
	MemSet(bmp,sizeof(BitmapType)+(rowBytes*height),0);
	bmp->width = width;
	bmp->height = height;
	bmp->rowBytes = rowBytes;
	bmp->pixelSize = 1;
	bmp->version = 1;
	tmpPtr = (unsigned char *) (bmp + 1);
	for (ypos=0;ypos<height;ypos++){
	  for (xpos=0;xpos<wbmpRowBytes;xpos++){
	    *(tmpPtr+xpos+(ypos*rowBytes))=(wsppdu->data[4+xpos+(ypos*wbmpRowBytes)]) ^ 0xFF ; // wbmp is inversed 
  	  }
	}
    prevDrawWin = WinGetDrawWindow();
	palmResizeWindow(height,g); // resize window buffer if needed 
    WinSetDrawWindow(g->DrawWin);
	WinDrawBitmap(bmp,(160-width)/2,g->y);
    WinSetDrawWindow(prevDrawWin);
    updateScreen(g);
	g->y+=height;
	if (bmp) {
    	Free(bmp);
    }
}

static void	drawTrigger(Int16 x, Int16 y)
// Draw the trigger symbol. I was surprised there is no special character in 
// the symbol charset for this, so I had to 'draw it by hand'
{
Int16 i;
	
	for(i=7; i>0; i-=2){  
		WinDrawLine(x,y,x+i,y);
		x++; y++;
	}
}

void palmHyperLink(Boolean ison, Char *prefix, Char *url, GlobalsType *g)
{
  if (ison)
  {
	WinSetUnderlineMode(grayUnderline); // one of noUnderline, grayUnderline, solidUnderline.
	addHyperLink(g->x, g->y+g->ScrollY+LINEHEIGHT,g); // add hyperlink entry, set start x,y
  	if(url) // if url given, set url
		setHyperLinkUrl(prefix, url,g); 
  }
  else
  {
	WinSetUnderlineMode(noUnderline);
    setHyperLinkEnd(g->x, g->y+g->ScrollY+24,g); 
	if(url) // if url given, set url
		setHyperLinkUrl(prefix, url,g);
  }
}

void palmWmlInputType(Boolean pwdType, GlobalsType *g)
{
	if(g->Input)
		g->Input->passwordType= pwdType;
}		

void palmWmlInputValue(Char *value, GlobalsType *g)
{
	if(g->Input){
		setVar(g->Input->name,value,g);
		palmDrawChars(value, StrLen(value), g->Input->x1, g->Input->y1-LINEHEIGHT,g);
	}
}		

void palmWmlInput(Char *fieldName, GlobalsType *g)
{
 WinHandle   	prevDrawWin;
 RectangleType 	rect;
 InputPtr tmp;
 
    if ( g->x> 100)
		palmprintln(g);	
	tmp = Malloc (sizeof(InputType));
	ErrFatalDisplayIf (!tmp, "Malloc Failed");
	tmp->name = Malloc(StrLen(fieldName)+1);
	ErrFatalDisplayIf (!tmp->name, "Malloc Failed");
	StrCopy(tmp->name,fieldName);
	addVar(fieldName,g);
    tmp->x1=g->x+3;
    tmp->y1=(g->y)+(g->ScrollY)+LINEHEIGHT;
 	rect.topLeft.x = g->x+3;
	rect.extent.x = 56 ;
	tmp->x2= tmp->x1 + rect.extent.x;
	rect.topLeft.y = g->y+g->ScrollY;
	rect.extent.y = LINEHEIGHT-1 ;
	tmp->y2= tmp->y1 + LINEHEIGHT-1 ;
	tmp->next = g->Input;
    g->Input = tmp;

    prevDrawWin = WinGetDrawWindow();
    WinSetDrawWindow(g->DrawWin);
	WinDrawGrayRectangleFrame(simpleFrame,&rect); // simpleFrame/rectangleFrame
    WinSetDrawWindow(prevDrawWin);
    updateScreen(g);
	palmprintln(g);	
	
}

void palmWmlSelectOptionTxt(Char *txt, GlobalsType *g)
{
 OptionPtr tmp;

	tmp = g->Select->options;
	if(tmp != NULL){
		while(tmp->next != NULL){
			tmp = tmp->next;
		}
		tmp->txt = Malloc(StrLen(txt)+1);
		ErrFatalDisplayIf (!tmp->txt, "Malloc Failed");
		StrCopy(tmp->txt,txt);
	}
		
}

void palmWmlSelect(Char *name, GlobalsType *g)
{
 SelectPtr 		tmp;
 
    if ( g->x> 100)
		palmprintln(g);	
	tmp = Malloc (sizeof(SelectType));
	ErrFatalDisplayIf (!tmp, "Malloc Failed");
	tmp->name = Malloc(StrLen(name)+1);
	ErrFatalDisplayIf (!tmp->name, "Malloc Failed");
	StrCopy(tmp->name,name);
	addVar(name,g);
    tmp->x1=g->x;
    tmp->y1=g->y+g->ScrollY+LINEHEIGHT;
	tmp->x2= tmp->x1 + 10;
	tmp->y2= tmp->y1 + LINEHEIGHT-1;
	tmp->next = g->Select;
	tmp->options = NULL;
    g->Select = tmp;

}

void palmWmlSelectValue(Char *ivalue, GlobalsType *g)
{
	if(g->Select){
		setVar(g->Select->name,ivalue,g);
	}
}

void palmWmlSelectOption(Char *value, GlobalsType *g)
{
 OptionPtr tmp,nw;

	if(g->Select){
		nw = Malloc (sizeof(OptionType));
		ErrFatalDisplayIf (!nw, "Malloc Failed");
		nw->value = Malloc(StrLen(value)+1);
		ErrFatalDisplayIf (!nw->value, "Malloc Failed");
		StrCopy(nw->value,value);
		nw->txt = NULL;
		nw->next = NULL;
		tmp = g->Select->options;
		if(tmp == NULL){
			g->Select->options = nw;
		}else{
			while(tmp->next != NULL){
				tmp = tmp->next;
			}
			tmp->next = nw;
		}
	}
}
 
void selectDrawText(SelectPtr slct, GlobalsType *g)
{
 Char	*txt,*value;
 OptionPtr tmp;
 
 if(slct){
 	if(slct->name){
		value = getVar(slct->name,g);
		tmp = slct->options;
		txt=NULL;
		while(tmp != NULL)
		{
			if (StrCompare(value,tmp->value)==0)
				txt=tmp->txt;
			tmp=tmp->next;
		}
		if (txt)
			WinDrawChars(txt, StrLen(txt), slct->x1+18, slct->y1+g->ScrollY-LINEHEIGHT);
	}		
 }
}


void palmWmlSelectEnd(GlobalsType *g)
{
 WinHandle  prevDrawWin;
 
 prevDrawWin = WinGetDrawWindow();
 WinSetDrawWindow(g->DrawWin);
 drawTrigger(g->x+3,g->y+g->ScrollY+4);
 selectDrawText(g->Select,g);
 palmprintln(g);	
 WinSetDrawWindow(prevDrawWin);
 updateScreen(g);
}

void palmUnderline(Boolean ison)
{
if (ison)
	WinSetUnderlineMode(solidUnderline); // one of noUnderline, grayUnderline, solidUnderline.
else
	WinSetUnderlineMode(noUnderline);
}

void palmBold(Boolean ison)
{
if (ison)
	FntSetFont(boldFont);
else
	FntSetFont(stdFont);
}

void palmDrawChars(char *str, Int16 textLen, Int16 x, Int16 y, GlobalsType *g)
{
	WinHandle	prevDrawWin;

    prevDrawWin = WinGetDrawWindow();
	palmResizeWindow(LINEHEIGHT,g); // resize window buffer if needed
	WinSetDrawWindow(g->DrawWin);
    WinDrawChars(str,textLen,x,y);
    WinSetDrawWindow(prevDrawWin);
    updateScreen(g);
}

void palmprintf(char *str, GlobalsType *g)
{
    Int16         textLen, width, wp;
    Boolean     fits;

    width = 152-g->x;
    textLen = StrLen(str);
    FntCharsInWidth(str, &width, &textLen, &fits);
    while (!fits) {
        // check if there is a space-char so we we can wrap
        wp= WrapPos(str,textLen);
        if (wp == -1 ) {  // no space-character in string, try if newline would help
            width = 152;
            textLen = StrLen(str);
            FntCharsInWidth(str, &width, &textLen, &fits);
            if (!fits) { // the string still does not fit after a newline, check again for space-char
                wp= WrapPos(str,textLen);
                if (wp == -1) { // we'll need to cut ...
                    width = 152-g->x;
                    FntCharsInWidth(str, &width, &textLen, &fits);
                    palmDrawChars(str,textLen,g->x,g->y,g);
                    g->x+=FntCharsWidth(str,textLen);
                    str=str+textLen;                        
                } else { // there is a space-char at position 'wp'
                    palmprintln(g);
                    palmDrawChars(str,wp,g->x,g->y,g);
                    g->x+=FntCharsWidth(str,wp);
                    str=str+wp;
                }
            } else { // the complete string fits after a newline. 
                palmprintln(g);
                palmDrawChars(str,textLen,g->x,g->y,g);
                g->x+=FntCharsWidth(str,textLen);
                if (g->y<160) { // visible part, so update real screen
                    updateScreen(g);
                }
                return;
            }
        } else { // there is a space-char at position 'wp'
            palmDrawChars(str,wp,g->x,g->y,g);
            g->x+=FntCharsWidth(str,wp);
            str=str+wp;
        }
        width = 152-g->x;
        textLen = StrLen(str);
        FntCharsInWidth(str, &width, &textLen, &fits);                
    }
    palmDrawChars(str,textLen,g->x,g->y,g);
    g->x+=FntCharsWidth(str,textLen);
    if (g->y<160) { // visible part, so update real screen
        updateScreen(g);
    }
}

void palmsrcprintf(char *str, GlobalsType *g)
{
    Char * srcPtr;
    UInt32   sz;

    sz = MemHandleSize(g->srcHandle);
    srcPtr = MemHandleLock(g->srcHandle);
    if ( sz <= (StrLen(srcPtr)+StrLen(str))) { // We need to resize
        MemHandleUnlock(g->srcHandle);
        if (MemHandleResize(g->srcHandle,sz+250) == 0) {
            sz+=250;
            srcPtr = MemHandleLock(g->srcHandle);
        } else {
            DisplayError("Resize Failed");
            return; 
        }
    }
    StrCat(srcPtr,str);
    MemHandleUnlock(g->srcHandle);

}

Err OpenNetwork(GlobalsType *g)
{
    Err             error;
//    UInt16          InterfaceError;

    if (g->NetLibOpen==true)
        return(0);
    if (SysLibFind ("Net.lib", &AppNetRefnum) == 0) {
        error = NetLibOpen (AppNetRefnum, &errno);
        if (errno != 0) {
            MyErrorFunc("NetLibOpen",WspHostErrStr());
            NetLibClose (AppNetRefnum, true);
        } else if (error == 0 || error == netErrAlreadyOpen) {
            g->NetLibOpen= true;
            return(0);
        }
        return(error);
    } else {
        DisplayError ("PANIC: NO NETLIB FOUND!");   
        return(-1);
    }
}


void showProgress(GlobalsType *g)
{
    BitmapPtr resP=0;

    resP=MemHandleLock(g->resBmpPrgrs[g->progress]);
    WinDrawBitmap (resP, 44, 147);
    MemPtrUnlock(resP);
    g->progress= ((g->progress)+1)%4;
}

Boolean runBrowserState(GlobalsType *g)
{
    static UInt32			maxTimer;
    static Int16    		len;
    WinHandle       		prevDrawWin;
    static RectangleType 	r = {{0, 10}, {153, 135}};

    if (g->state==BS_IDLE) {
        return(true);
    }
    // Update progres indicator
    showProgress(g);
    switch (g->state) {
// 
    case BS_GET_URL:
        FrmSetTitle(FrmGetActiveForm(),"Requesting URL");
        g->sock = wapGetUrl (&(g->conn),g->Url,true,g);
        g->state = BS_GET_REPLY;
        FrmSetTitle(FrmGetActiveForm(),"Waiting for reply");
        maxTimer = TimGetTicks() + (REPLY_TIMEOUT * SysTicksPerSecond());
        break;
    case BS_GET_REPLY:
        len = WspGetReply(g->sock,g->inBuf,DEF_RECEIVE_BUF,maxTimer,g);
        break;
    case BS_WSP_DECODE:
        FrmSetTitle(FrmGetActiveForm(),"Decoding data");
        if (WspDecode(g->inBuf,len, &(g->wsppdu) ) == 0) {
            g->state = BS_WBXML_PARSE;
        } else {
            g->state = BS_IDLE;
        }
        break;
    case BS_WBXML_PARSE:		
		g->wmldck.card=NULL;
        FrmSetTitle(FrmGetActiveForm(),"Parsing data");
        wbxml_parse(&(g->wsppdu), ACTION_PARSE, &(g->wmldck), 0,g);
    	if(g->wmldck.card)  // if there is a first card defined, show it
	        g->CardOffset = g->wmldck.card->offset;
		else
	        g->CardOffset = 0;
		g->state= BS_RENDER;
        break;
	case BS_NEW_CARD:
	    g->x=0;g->y=0;
    	g->ScrollY=0;
	    disposeHyperLinks(g);
		disposeVar(g);
		disposeInput(g);
		disposeSelect(g);
	    prevDrawWin = WinGetDrawWindow();
    	WinSetDrawWindow(g->DrawWin);
   	 	WinEraseWindow();
    	WinSetDrawWindow(prevDrawWin);
    	WinEraseRectangle(&r,0);
    	scrollUpdate(g);
	    //OpenNetwork();
		g->state= BS_RENDER;
		break;
	case BS_RENDER:
        FrmSetTitle(FrmGetActiveForm(),"Rendering data");
		g->wmldck.ontimerUrl=NULL;
		g->wmldck.timerVal=-1;
	    wbxml_parse(&(g->wsppdu), ACTION_RENDER, &(g->wmldck), g->CardOffset,g);
        updateScreen(g);
		if(g->wmldck.timerVal != -1){
	        g->state = BS_ONTIMER;
	        maxTimer = TimGetTicks() + ( g->wmldck.timerVal * SysTicksPerSecond() );
		}else{
			g->state = BS_IDLE;
		}
        g->progress= 0;
        showProgress(g);
		break;	
	case BS_ONTIMER:
		if (maxTimer < TimGetTicks())
			followLink(g->wmldck.ontimerUrl,g); 
		break;
		
    default:
    }
    return true;
}

static void frmOpenEvent_OH(GlobalsType *g)
{
    Err             error;
    WinHandle       prevDrawWin;

    g->resBmpPrgrs[0]=DmGetResource(bitmapRsc, bmpProgress1);
    g->resBmpPrgrs[1]=DmGetResource(bitmapRsc, bmpProgress2);
    g->resBmpPrgrs[2]=DmGetResource(bitmapRsc, bmpProgress3);
    g->resBmpPrgrs[3]=DmGetResource(bitmapRsc, bmpProgress4);
    g->progress=0;
    g->srcHandle=0;
	g->sock = -1;
    g->inBuf = NULL;
    g->HyperLinks = NULL;
	g->Var = NULL;
	g->Input = NULL;
	g->Select = NULL;
	g->Url = NULL;
	g->InputFldActive=NULL;
    dbUrlReadCurrentUrl(g);
    FrmDrawForm(FrmGetActiveForm());
    prevDrawWin = WinGetDrawWindow();
    g->DrawWin = WinCreateOffscreenWindow(153,150,screenFormat,&error);
    g->DrawHeight=132;
    g->ScrollY=0;
    WinSetDrawWindow(g->DrawWin);
    WinEraseWindow();
    WinSetDrawWindow(prevDrawWin);
    SclSetScrollBar(GetObjectPtr(scrollBarBrowser),0,0,0,13);
    g->x=0;g->y=0;
    g->state = BS_IDLE;
    g->wsppdu.data= NULL;

    g->inBuf=Malloc(DEF_RECEIVE_BUF+1);   
	ErrFatalDisplayIf (!g->inBuf, "Malloc");
	
    dbConnGetConnection(g->DbUrlCurrentURL.connection,&(g->conn),g);
    g->Url=Malloc(StrLen(g->DbUrlCurrentURL.urlstr)+1);
    ErrFatalDisplayIf (!g->Url, "Malloc gUrl");
	StrCopy(g->Url,g->DbUrlCurrentURL.urlstr);  
	historyPush(g->Url,g);
	if(StrNCaselessCompare(g->Url,"MTCH:",5)==0)
	{
		g->state = BS_GET_OFFLINE;
	}else
	{
	    error = OpenNetwork(g);
	    if (error == 0) {
		    g->state = BS_GET_URL;
	    }
	} 
}


Boolean browserReloadEvt(GlobalsType *g)
{
    Err             error;
    WinHandle       prevDrawWin;
    RectangleType r = {{0, 10}, {153, 135}};

    g->x=0;g->y=0;
    g->ScrollY=0;

	g->InputFldActive=NULL;
    WspFreePdu(&(g->wsppdu));
    disposeHyperLinks(g);
	disposeVar(g);
	disposeInput(g);
	disposeSelect(g);
    wbxml_free_wmldck(&(g->wmldck));
// 
	    error = OpenNetwork(g);
	    if (error == 0) {
		    g->state = BS_GET_URL;
	    }
// 
    prevDrawWin = WinGetDrawWindow();
    WinSetDrawWindow(g->DrawWin);
    WinEraseWindow();
    WinSetDrawWindow(prevDrawWin);
    WinEraseRectangle(&r,0);
    scrollUpdate(g);
    return(true);   
}

Boolean browserSrcEvt(GlobalsType *g)
{
    WMLDeck         wmldck;

    if (g->wsppdu.data==NULL) {
        MyErrorFunc ("No page loaded", "View Source");
        return(true);
    }
    g->srcHandle = MemHandleNew(20);
    if (g->srcHandle<=0) {
        MyErrorFunc ("MemHandleNew Failed", "ButtonSrc_OH");
        return(true);
    }
    MemSet(MemHandleLock(g->srcHandle),20,0);
    MemHandleUnlock(g->srcHandle);
	wmldck.card=NULL;
    wbxml_parse(&(g->wsppdu), ACTION_SRC, &wmldck,0,g );
    wbxml_free_wmldck(&wmldck);
    ScrollFrm("WML Source",g->srcHandle);
    MemHandleFree(g->srcHandle);
    g->srcHandle=0;
    return(true);   
}

static void frmCloseEvent_OH(GlobalsType *g)
{

    // close socket if needed              
    if (g->sock >= 0)
        close (g->sock);
	if(g->NetLibOpen==true)
	    NetLibClose (AppNetRefnum, false);
   	g->NetLibOpen=false;
    if (g->inBuf != NULL) {
        Free(g->inBuf);
        g->inBuf = NULL;
    }
    if (g->Url != NULL) {
        Free(g->Url);
        g->Url = NULL;
    }
    WinDeleteWindow(g->DrawWin,false);
    g->DrawWin = NULL;
    WspFreePdu(&(g->wsppdu));
    disposeHyperLinks(g);
	disposeVar(g);
	disposeInput(g);
	disposeSelect(g);
    wbxml_free_wmldck(&(g->wmldck));
    DmReleaseResource(g->resBmpPrgrs[0]);
    DmReleaseResource(g->resBmpPrgrs[1]);
    DmReleaseResource(g->resBmpPrgrs[2]);
    DmReleaseResource(g->resBmpPrgrs[3]);
}


/**----------------------------------**/
/** Form Event Handler for FormBrowser**/
/**----------------------------------**/

/****************************************************/
/* 
   Function: Boolean FormBrowser_EH(EventPtr event);
   Description: Main Form Event Handler for FormBrowser
 */
/****************************************************/


/* Form Event Handler */
Boolean ehFormBrowser(EventPtr event)
{
    Boolean handled = false;
  	GlobalsType *g;


#ifdef __GNUC__
    CALLBACK_PROLOGUE
#endif
	FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);
    switch (event->eType) {
    
    case ctlSelectEvent:
        if (event->data.ctlSelect.controlID == buttonBack) {
			deactivateInputFld(g);
            handled = browserBackEvt(g);
        }
        break;

    case frmOpenEvent:
        frmOpenEvent_OH(g);
        break;

    case frmCloseEvent:
        frmCloseEvent_OH(g);
        break;

    case nilEvent:
        handled = runBrowserState(g);
        break;

    case sclRepeatEvent:
		deactivateInputFld(g);
        g->ScrollY = event->data.sclRepeat.newValue*10;
        scrollUpdate(g);
        break;

    case keyDownEvent:
        switch (event->data.keyDown.chr) {
        case pageUpChr:
			deactivateInputFld(g);
            scrollUp(g);
            break;

        case pageDownChr:
			deactivateInputFld(g);
			scrollDown (g);
            break;
        }
        break;

	case penUpEvent: // leave this one at the end of the switch !!
		handled = browserPenUpEvt(event,g);
		break;

    case menuEvent:
		deactivateInputFld(g);
        handled = mhMenuView(event->data.menu.itemID,g);
  	    break;
		
    default:
        break;		

    }
#ifdef __GNUC__
    CALLBACK_EPILOGUE
#endif
    return handled;
}


/* Item Handler Functions Here */

/**--------------------------------------**/
/** Form Object Handler Functions for FormBrowser **/
/**--------------------------------------**/



/****************************************************/
/* 
   Function: Boolean ButtonURLs_OH(EventPtr event);
   Description: Button Object Handler for the FormBrowser Form
 */
/****************************************************/

static Boolean browserPenUpEvt(EventPtr event, GlobalsType *g)
{
Int16 xpos,ypos;

	xpos = event->screenX;
	ypos = event->screenY;
	if((xpos > 153) || (ypos > 144))
		return(false);
	// user clicked in the browser area, check if he clicked an hyperlink
	ypos += g->ScrollY;
	// let's see if we have a link under this xpos/ypos position
	if (!checkSelect(xpos,ypos,g)){
		if(!checkInput(xpos,ypos,g)){
			return(checkHyperLink(xpos, ypos,g));
		}
	}
	return(true);
}

Boolean browserStopEvt(GlobalsType *g)
{
    g->progress= 0;
    showProgress(g);
    g->state = BS_IDLE;
    FrmSetTitle(FrmGetActiveForm(),"WAPUniverse");
    return(true);
}

			
/****************************************************/
/* 
   Function: Boolean ButtonURLs_OH(EventPtr event);
   Description: Button Object Handler for the FormBrowser Form
 */
/****************************************************/

Boolean browserURLsEvt(GlobalsType *g)
{
    SwitchForm(frmUrlList,g);
    return(true);   
}


/****************************************************/
/* 
   Function: Boolean ButtonBack_OH(EventPtr event);
   Description: Button Object Handler for the FormBrowser Form
 */
/****************************************************/

Boolean browserBackEvt(GlobalsType *g)
{

Char *tmpUrl;

	tmpUrl = historyPop(g);
	if (!tmpUrl)
		return(true);
	if(g->Url)
		Free(g->Url);
    g->Url=Malloc(MAX_URL_LEN+1);
	ErrFatalDisplayIf (!g->Url, "Malloc");
    wapCreateUrlStr(tmpUrl,g->Url,g);
	free(tmpUrl);
	browserReloadEvt(g);
    return(true);   
}



