// ---------------------------------------------------------------------------
// FormConnectionList.c
// Contains the PalmOS specific code for the ConnectionList form
// 
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
// 
// $Workfile: formConnectionList.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.7 $
// 
// $Header: /cvsroot/wapuniverse/wapuniverse/src/formConnectionList.c,v 1.7 2000/11/11 20:51:09 wapuniverse Exp $
// 
// -------------------------------------------------------------------------------
#include    <PalmOS.h>
#include    <SysEvtMgr.h>
#include 	"WAPUniverse.h"
#include 	"../res/WAPUniverse_res.h"
#include    "formConnectionList.h"
#include    "MenuHnd.h"
#include    "dbConn.h"
#include 	"WAP.h"
#include 	"PalmUtils.h"
#ifdef __GNUC__
#include "callbacks.h"
#endif

		

// function prototypes
static Boolean ButtonConnectionActivate_OH(EventPtr event, GlobalsType *g);
static Boolean ButtonConnectionNew_OH(EventPtr event, GlobalsType *g);
static Boolean ListConnectionSettings_OH(EventPtr event);
static Boolean ButtonConnectionEdit_OH(EventPtr event, GlobalsType *g);

void enableConnectionEditOpen(GlobalsType *g)
{
  FormPtr	frm;
  Boolean 	enable;
  UInt16    nr;
  
    nr =  dbConnNumRecords(g); 
    if (nr >0)
		enable = true;
	else
		enable = false;
    frm = FrmGetFormPtr (frmConnectionList);
    CtlSetEnabled(FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, buttonConnectionActivate)),enable);
  	CtlSetEnabled(FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, buttonConnectionEdit)),enable);
    CtlSetUsable(FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, buttonConnectionActivate)),enable);
  	CtlSetUsable(FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, buttonConnectionEdit)),enable);
	
}


static void
DrawOneConnectionInList (Int16 itemNumber, RectangleType *bounds, Char * * text)
{
  MemHandle        	h;
  Int16             seekAmount = itemNumber;
  UInt16            index = 0;
  GlobalsType 		*g;


#ifdef __GNUC__
  CALLBACK_PROLOGUE
#endif

 FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);

  // must do seek to skip over secret records
  DmSeekRecordInCategory (g->DbConnDb, &index, seekAmount, dmSeekForward, dmAllCategories);
  h = DmQueryRecord (g->DbConnDb, index);
  if (h)
    {
      dbConnPackedConnection* packedConnection = MemHandleLock (h);

      if (StrCompare (packedConnection->name, g->ActiveConnection) == 0)
        DrawCharsToFitWidth (packedConnection->name, bounds, '\273');
      else
        DrawCharsToFitWidth (packedConnection->name, bounds, ' ');
      MemHandleUnlock (h);
    }
#ifdef __GNUC__
  CALLBACK_EPILOGUE
#endif
}

static void
InitViewConnectionList(GlobalsType *g)
{

  FormPtr         frm = FrmGetActiveForm ();
  ListPtr         list = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, lstConnectionSettings));
  UInt16            numConnections = DmNumRecordsInCategory (g->DbConnDb, dmAllCategories);
  LstSetListChoices (list, NULL, numConnections);
  LstSetDrawFunction (list, DrawOneConnectionInList);

}


/**----------------------------------**/
/** Form Event Handler for FormConnectionList**/
/**----------------------------------**/

/****************************************************/
/* 
   Function: Boolean FormConnectionList_EH(EventPtr event);
   Description: Main Form Event Handler for FormConnectionList
 */
/****************************************************/


/* Form Event Handler */
Boolean ehFormConnectionList(EventPtr event)
{
    Boolean handled = false;
	GlobalsType *g;
	
#ifdef __GNUC__
    CALLBACK_PROLOGUE
#endif

	FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);
    switch(event->eType) {

        case ctlSelectEvent:
            if (event->data.ctlSelect.controlID == buttonConnectionActivate) {
                handled = ButtonConnectionActivate_OH(event, g); }
            if (event->data.ctlSelect.controlID == buttonConnectionNew) {
                handled = ButtonConnectionNew_OH(event, g); }
            if (event->data.ctlSelect.controlID == buttonConnectionEdit) {
                handled = ButtonConnectionEdit_OH(event, g); }
            if (event->data.ctlSelect.controlID == buttonViewC) {
				if(dbConnNumRecords(g)>0){
                    SwitchForm(frmUrlList,g); 					
				}
				else{
					FrmCustomAlert (alError, "You need to define at least one connection\n", "", NULL);
				}
				}
            if (event->data.ctlSelect.controlID == buttonInfoC) {
                DisplayAbout(); }
            break;

        case ctlRepeatEvent:
            break;

        case tblSelectEvent:
            break;

        case lstSelectEvent:
        case popSelectEvent:
            if (event->data.ctlSelect.controlID == lstConnectionSettings) {
                ListConnectionSettings_OH(event); }
            break;

        case sclEnterEvent:
            break;


        case frmOpenEvent:
           InitViewConnectionList(g);
	       enableConnectionEditOpen(g);
           FrmDrawForm(FrmGetActiveForm());
           handled = true;
           break;

         case menuEvent:
             mhMenuOptions(event->data.menu.itemID,g);
    	     break;
	default:
	     break;


           }
#ifdef __GNUC__
    CALLBACK_EPILOGUE
#endif
    return handled;
}


/* Item Handler Functions Here */

/**--------------------------------------**/
/** Form Object Handler Functions for FormConnectionList **/
/**--------------------------------------**/

/****************************************************/
/* 
   Function: Int16 ButtonConnectionActivate_OH(EventPtr event);
   Description: Button Object Handler for the FormConnectionList Form
 */
/****************************************************/

static Boolean ButtonConnectionActivate_OH(EventPtr event, GlobalsType *g)
{
  FormPtr frm;

     frm = FrmGetActiveForm ();
     g->DbConnCurrentRecord = LstGetSelection (FrmGetObjectPtr (frm,                                                           FrmGetObjectIndex
                                                                (frm, lstConnectionSettings)));
     dbConnReadCurrentConnection (g);
     StrCopy (g->ActiveConnection, g->CurrentConnection.name);
     g->CurrentView = frmConnectionList;
     FrmGotoForm (g->CurrentView);
 	 return(true);
}


/****************************************************/
/* 
   Function: Int16 ButtonConnectionNew_OH(EventPtr event);
   Description: Button Object Handler for the FormConnectionList Form
 */
/****************************************************/
static Boolean ButtonConnectionNew_OH(EventPtr event, GlobalsType *g)
{

   dbConnCreateRecord (g);
   g->CurrentView = frmEditConnection;
   FrmGotoForm (g->CurrentView);
   enableConnectionEditOpen(g);
   return(true);
}


/****************************************************/
/* 
   Function: Int16 ButtonConnectionEdit_OH(EventPtr event);
   Description: Button Object Handler for the FormConnectionList Form
 */
/****************************************************/
static Boolean ButtonConnectionEdit_OH(EventPtr event, GlobalsType *g)
{
  FormPtr frm;
  
    frm = FrmGetActiveForm ();
    g->DbConnCurrentRecord = LstGetSelection (FrmGetObjectPtr (frm, 
                           FrmGetObjectIndex(frm, lstConnectionSettings)));
    g->CurrentView = frmEditConnection;
    FrmGotoForm (g->CurrentView);
	return(true);
}


/****************************************************/
/* 
   Function: Int16 ListConnectionSettings_OH(EventPtr event);
   Description: List Object Handler for the FormConnectionList Form
 */
/****************************************************/

static Boolean ListConnectionSettings_OH(EventPtr event)
{

	return(0);
}


