// ---------------------------------------------------------------------------
// FormEditURL.c
// Contains PalmOS specific code, for EditURL form
// 
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
// 
// $Workfile: formEditURL.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.9 $
// 
// $Header: /cvsroot/wapuniverse/wapuniverse/src/formEditURL.c,v 1.9 2000/11/11 20:51:09 wapuniverse Exp $
// 
// -------------------------------------------------------------------------------#include "Decls.h"
#include    <PalmOS.h>
#include    <SysEvtMgr.h>
#include 	"WAPUniverse.h"
#include    "formConnectionList.h"
#include    "dbUrl.h"
#include    "dbConn.h"
#include    "MenuHnd.h"
#include    "PalmUtils.h"
#include    "FormEditUrl.h"
#include 	"../res/WAPUniverse_res.h"
#ifdef __GNUC__
#include "callbacks.h"
#endif



static Boolean EditURLDoneButton_OH(EventPtr event, GlobalsType *g) SEC_1;
static Boolean TriggerURLConnection_OH(EventPtr event, GlobalsType *g) SEC_1;
static Boolean EditURLCategoryPoptrigger_OH(EventPtr event, GlobalsType *g) SEC_1;
static Boolean EditURLPrivateCheckBox_OH(EventPtr event) SEC_1;
static Boolean EditURLDeleteButton_OH(EventPtr event, GlobalsType *g) SEC_1;

static void URLDetailsInit(GlobalsType *g)
{
  ControlPtr      ctl;
  UInt16          attr;
  Boolean         secret;
  dbUrlPackedURL* purl;
  MemHandle       rechand;
  Int16           seekAmount = g->DbUrlCurrentRecord;
  UInt16          index = 0;

  // must do seek to skip over secret records
  DmSeekRecordInCategory (g->DbUrlDb, &index, seekAmount, dmSeekForward, g->DbUrlCurrentCategory);

  // Get the secret and category attributes of the current record.
  DmRecordInfo (g->DbUrlDb, index, &attr, NULL, NULL);
  secret = attr & dmRecAttrSecret;
  g->RecordCategory = attr & dmRecAttrCategoryMask;

  // If the record is marked secret, turn on the secret checkbox.
  CtlSetValue (GetObjectPtr (chkbxEditURLPrivate), secret);

  rechand = DmQueryRecord (g->DbUrlDb, index);
  purl = MemHandleLock (rechand);
  dbUrlUnPackURL (&g->DbUrlCurrentURL, purl);

  // Set the label of the category trigger.
  CategoryGetName (g->DbUrlDb, g->RecordCategory, g->DbUrlCategoryName);
  ctl = GetObjectPtr (poptriggerEditURLCategory);
  CategorySetTriggerLabel (ctl, g->DbUrlCategoryName);

  // NewCategory indicates which category wants to changed the 
  // current record to.  Initialize it to the current record's current
  // category.
  g->NewCategory = g->RecordCategory;
  g->NewCategoryEdited = false;

  SetFieldFromStr(g->DbUrlCurrentURL.name , fldURLTitle);
  if (StrLen(g->DbUrlCurrentURL.urlstr) == 0)
  	StrCopy(g->DbUrlCurrentURL.urlstr,"http://");
  SetFieldFromStr(g->DbUrlCurrentURL.urlstr , fldURL);

  MemHandleUnlock (rechand);
  dbConnGetConnectionsList(g);

}

static UInt16 URLDetailsApply (GlobalsType *g)
{
  UInt16        attr;
  Boolean       secret;
  UInt16        updateCode = 0;
  Boolean       dirty = false;    // P11. If an attribute changed
  dbUrlURL     	tURL;
  MemHandle     currec;
  FormPtr       frm;
  FieldPtr      ipPtr;
  Int16         seekAmount = g->DbUrlCurrentRecord;
  UInt16        index = 0;
  Char 			*c;

  // must do seek to skip over secret records
  DmSeekRecordInCategory (g->DbUrlDb, &index, seekAmount, dmSeekForward, g->DbUrlCurrentCategory);

  // Get the secret and category attributes of the current record.
  DmRecordInfo (g->DbUrlDb, index, &attr, NULL, NULL);

  frm = FrmGetFormPtr (frmEditURL);
  ipPtr = (FieldPtr) (FrmGetObjectPtr (frm, (FrmGetObjectIndex (frm, fldURLTitle))));
  if (FldGetTextPtr (ipPtr) != NULL)
    StrCopy (tURL.name, FldGetTextPtr (ipPtr));
  ipPtr = (FieldPtr) (FrmGetObjectPtr (frm, (FrmGetObjectIndex (frm, fldURL))));
  if (FldGetTextPtr (ipPtr) != NULL){
    StrCopy (tURL.urlstr, FldGetTextPtr (ipPtr));
	while((c=StrChr(tURL.urlstr,0x0A))){  // remove CR/LF
		StrCopy(c,c+1);
	}
  }
  StrCopy (tURL.connection,g->EditURLConnection);

  currec = DmGetRecord (g->DbUrlDb, index);
  dbUrlPackURL (&tURL, currec);
  DmReleaseRecord (g->DbUrlDb, index, true);

  // Get the current setting of the secret checkbox.
  secret = (CtlGetValue (GetObjectPtr (chkbxEditURLPrivate)) != 0);

  // Has the secret attribute been changed?
  if (((attr & dmRecAttrSecret) == dmRecAttrSecret) != secret)
    {
      // Yes, it was changed.  What is the new setting?
      if (secret)
        // Now it should be secret.
        attr |= dmRecAttrSecret;
      else
        // Now it should not be secret.
        attr &= ~dmRecAttrSecret;

      dirty = true;
    }


  // P11. Has the category been changed?
  if (g->RecordCategory != g->NewCategory)
    {
      attr &= ~dmRecAttrCategoryMask; // Remove all category bits.
      attr |= g->NewCategory;      // Set the new category
      dirty = true;             // mark this record dirty later
      g->RecordCategory = g->NewCategory;     // remember which category the record 
                                        // is in
      updateCode |= updateCategoryChanged;  // send this update code
    }


  // P11. If the current category was deleted, renamed, or merged with
  // another category, then the list view needs to be redrawn.
  if (g->NewCategoryEdited)
    {
      g->RecordCategory = g->NewCategory;
      updateCode |= updateCategoryChanged;
    }


  if (dirty)
    {
      // Set the dirty flag for the record.
      attr |= dmRecAttrDirty;
      DmSetRecordInfo (g->DbUrlDb, index, &attr, NULL);
    }

  dbUrlSortRecord (g->DbUrlDb, &index);

  return updateCode;
}


/**----------------------------------**/
/** Form Event Handler for FormEditURL**/
/**----------------------------------**/

/****************************************************/
/* 
   Function: Boolean FormEditURL_EH(EventPtr event);
   Description: Main Form Event Handler for FormEditURL
 */
/****************************************************/


/* Form Event Handler */
Boolean ehFormEditURL(EventPtr event)
{
    Boolean handled = false;
	GlobalsType *g;
	
#ifdef __GNUC__
    CALLBACK_PROLOGUE
#endif

  	FtrGet(wuAppType, ftrGlobals, (UInt32 *)&g);
    switch(event->eType) {

        case ctlSelectEvent:
            if (event->data.ctlSelect.controlID == buttonEditURLDone) {	
                handled = EditURLDoneButton_OH(event,g); }
            if (event->data.ctlSelect.controlID == buttonEditURLDelete) {
                handled = EditURLDeleteButton_OH(event,g); }
            if (event->data.ctlSelect.controlID == chkbxEditURLPrivate) {
                handled = EditURLPrivateCheckBox_OH(event); }
            if (event->data.ctlSelect.controlID == poptriggerEditURLCategory) {
                handled = EditURLCategoryPoptrigger_OH(event,g); }
			break;

        case lstSelectEvent:
        case popSelectEvent:
            if (event->data.popSelect.controlID == poptriggerEditURLCategory) {
                handled = EditURLCategoryPoptrigger_OH(event,g); }
            if (event->data.popSelect.controlID == triggerURLConnection) {
                handled = TriggerURLConnection_OH(event,g); }
            break;

        case frmOpenEvent:
           URLDetailsInit(g);
           FrmDrawForm(FrmGetActiveForm());
           handled = true;
           break;

        case menuEvent:
           mhMenuEdit(event->data.menu.itemID);
    	   break;
	default:
	   break;


           }
#ifdef __GNUC__
    CALLBACK_EPILOGUE
#endif
    return handled;
}


/* Item Handler Functions Here */

/**--------------------------------------**/
/** Form Object Handler Functions for FormEditURL **/
/**--------------------------------------**/

/****************************************************/
/* 
   Function: Int16 EditURLDoneButton_OH(EventPtr event);
   Description: Button Object Handler for the FormEditURL Form
 */
/****************************************************/

static Boolean EditURLDoneButton_OH(EventPtr event, GlobalsType *g)
{
    URLDetailsApply(g);
    dbConnDisposeConnectionsList(g);
    FrmCloseAllForms ();
    FrmGotoForm (frmUrlList);
	return(true);
}


/****************************************************/
/* 
   Function: Int16 EditURLDeleteButton_OH(EventPtr event);
   Description: Button Object Handler for the FormEditURL Form
 */
/****************************************************/

static Boolean EditURLDeleteButton_OH(EventPtr event, GlobalsType *g)
{
   if (dbUrlDeleteCurrentRecord (g))
   {
       // Delete was completed.  Close the details and the edit forms.
       dbConnDisposeConnectionsList(g);
       FrmCloseAllForms ();

       // Go to the main form.
       FrmGotoForm (frmUrlList);
   }
   return(true);
}


/****************************************************/
/* 
   Function: Int16 EditURLPrivateCheckBox_OH(EventPtr event);
   Description: CheckBox Object Handler for the FormEditURL Form
 */
/****************************************************/

static Boolean EditURLPrivateCheckBox_OH(EventPtr event)
{

	return(0);
}


/****************************************************/
/* 
   Function: Int16 EditURLCategoryPoptrigger_OH(EventPtr event);
   Description: Pop-up Trigger Object Handler for the FormEditURL Form
 */
/****************************************************/

static Boolean EditURLCategoryPoptrigger_OH(EventPtr event, GlobalsType *g)
{
          if (CategorySelect (g->DbUrlDb, FrmGetActiveForm (),
                              poptriggerEditURLCategory, listEditURLCategoriesList,
                              true, &(g->NewCategory), g->DbUrlCategoryName, 1, 0))
            {
              g->NewCategoryEdited = true;
            }
  
	return(true);
}


/****************************************************/
/* 
   Function: Int16 TriggerURLConnection_OH(EventPtr event);
   Description: Pop-up Trigger Object Handler for the FormEditURL Form
 */
/****************************************************/

static Boolean TriggerURLConnection_OH(EventPtr event, GlobalsType *g)
{
    ControlPtr      ctlPtr;
    FormPtr         frm;

    g->EditURLConnection = LstGetSelectionText(event->data.popSelect.listP,
                            event->data.popSelect.selection); 
    frm = FrmGetFormPtr (frmEditURL);
    ctlPtr =(ControlPtr)(FrmGetObjectPtr(frm,(FrmGetObjectIndex(frm,triggerURLConnection))));
    CtlSetLabel(ctlPtr,g->EditURLConnection);
	return(true);
}


