//---------------------------------------------------------------------------
// wml.c
// Contains all platform independant WAP/WML code 
//
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
//
// $Workfile: wml.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.19 $
//
// $Header: /cvsroot/wapuniverse/wapuniverse/src/wml.c,v 1.19 2000/11/11 20:51:09 wapuniverse Exp $
//
//---------------------------------------------------------------------------

#include "wml.h"
#include "WAP.h"
#include "formBrowser.h"
#include "WAPUniverse.h"
#include "wsp.h"

#ifndef __palmos__
    #include <stdio.h>
#endif __palmos__

static char *wmltag[] = { 
    "-", // 05
    "-", // 06
    "-", // 07
    "-", // 08
    "-", // 09
    "-", // 0A
    "-", // 0B
    "-", // 0C
    "-", // 0D
    "-", // 0E
    "-", // 0F
    "-", // 10
    "-", // 11
    "-", // 12
    "-", // 13
    "-", // 14
    "-", // 15
    "-", // 16
    "-", // 17
    "-", // 18
    "-", // 19
    "-", // 1A
    "-", // 1B
    "a",  // 1C
    "td", // 1D
    "tr", // 1E
    "table", // 1F
    "p", // 20
    "postfield", // 21
    "anchor", // 22
    "access", // 23
    "b",  // 24
    "big", // 25
    "br", // 26
    "card", // 27
    "do", // 28
    "em", // 29
    "fieldset", // 2A
    "go", // 2B
    "head", // 2C
    "i", // 2D
    "img", // 2E
    "input", // 2F
    "meta", // 30
    "noop", // 31
    "prev", // 32
    "onevent", // 33
    "optgroup", // 34
    "option", // 35
    "refresh", // 36
    "select", // 37
    "small", // 38
    "strong", // 39
    "-", // 3A
    "template", // 3B
    "timer", // 3C
    "u", // 3D
    "setvar", // 3E
    "wml" // 3F
};

static char *attrStart[] = { 
    "accept-charset", // 05
    "align=\"bottom\"", // 06
    "align=\"center\"", // 07
    "align=\"left\"", // 08
    "align=\"middle\"", // 09
    "align=\"right\"", // 0A
    "align=\"top\"", // 0B
    "alt=\"", // 0C
    "content", // 0D
    "-", // 0E
    "domain", // 0F

    "emptyok=\"false\"", // 10
    "emptyok=\"true\"", // 11
    "=\"", // 12
    "height=\"", // 13
    "hspace=\"", // 14
    "ivalue", // 15
    "iname", // 16
    "-", // 17
    "label=\"", // 18
    "localsrc", // 19
    "maxlength", // 1A
    "method=\"get\"", // 1B
    "method=\"post\"", // 1C
    "mode=\"nowrap\"", // 1D
    "mode=\"wrap\"", // 1E
    "multiple=\"false\"", // 1F

    "multiple=\"true\"", // 20
    "name=\"", // 21
    "newcontext=\"false\"", // 22
    "newcontext=\"true\"", // 23
    "onpick", // 24
    "onenterbackward", // 25
    "onenterforward", // 26
    "ontimer", // 27
    "optimal=\"false\"", // 28
    "optimal=\"true\"", // 29
    "path", // 2A
    "-", // 2B
    "-", // 2C
    "-", // 2D
    "scheme", // 2E
    "sendreferer=false", // 2F

    "sendreferer=true", // 30
    "size", // 31
    "src=\"", // 32
    "ordered=\"true\"", // 33
    "ordered=\"false\"", // 34
    "tabindex", // 35
    "title=\"", // 36
    "type", // 37
    "type=\"accept\"", // 38
    "type=\"delete\"", // 39
    "type=\"help\"", // 3A
    "type=\"password\"", // 3B
    "type=\"onpick\"", // 3C
    "type=\"onenterbackward\"", // 3D
    "type=\"onenterforward\"", // 3E
    "type=\"ontimer\"", // 3F

    "-", // 40
    "-", // 41
    "-", // 42
    "-", // 43
    "-", // 44
    "type=\"options\"", // 45
    "type=\"prev\"", // 46
    "type=\"reset\"", // 47
    "type=\"text\"", // 48
    "type=\"vnd.", // 49
    "href=\"", // 4A
    "href=\"http://", // 4B
    "href=\"https://", // 4C
    "value=\"", // 4D
    "vspace=\"", // 4E
    "width=\"", // 4F

    "xml:lang", // 50
    "-", // 51
    "align", // 52
    "columns", // 53
    "class", // 54
    "id=\"", // 55
    "forua=\"false\"", // 56
    "forua=\"true\"", // 57
    "src=\"http://", // 58
    "src=\"https://", // 59
    "http-equiv", // 5A
    "http-equiv=\"Content-Type\"", // 5B
    "content=\"application/vnd.wap.wmlc;charset=", // 5C
    "http-equiv=\"Expires\"", // 5D
    "-", // 5E
    "-", // 5F
};


static char *attrVal[] = { 
    ".com/", // 85
    ".edu/", // 86
    ".net/", // 87
    ".org/", // 88
    "accept", // 89
    "bottom", // 8A
    "clear", // 8B
    "delete", // 8C
    "help", // 8D
    "http://", // 8E
    "http://www.", // 8F

    "https://", // 90
    "https://www.", // 91
    "?", // 92
    "middle", // 93
    "nowrap", // 94
    "onpick", // 95
    "onenterbackward", // 96
    "onenterforward", // 97
    "ontimer", // 98
    "options", // 99
    "password", // 9A
    "reset", // 9B
    "?", // 9C
    "text", // 9D
    "top", // 9E
    "unknown", // 9F
    "wrap", // A0
    "www.", // A1
};

char *getAttributes(WMLDeckPtr wmldck, unsigned char attr, GlobalsType *g);

#ifdef __palmos__
#define underline palmUnderline
#define bold palmBold
#define hyperLink palmHyperLink
#define wmlInput palmWmlInput
#define wmlInputValue palmWmlInputValue
#define getPos palmGetPos
#define wmlSelectOptionTxt palmWmlSelectOptionTxt
#define wmlSelect palmWmlSelect 
#define wmlSelectValue palmWmlSelectValue
#define wmlSelectOption palmWmlSelectOption
#define wmlSelectEnd palmWmlSelectEnd
#endif __palmos__

int pushwmlstack( WMLDeckPtr wmldck)
{
    unsigned char *ptr;
    int i;

    if (wmldck->stack == NULL) {
        wmldck->maxstack = 10;
        wmldck->stack=Malloc(wmldck->maxstack);
	    ErrFatalDisplayIf (!wmldck->stack, "Malloc(stack)");
    }
    if (wmldck->stackDepth >= wmldck->maxstack) {   	
        i = wmldck->maxstack;
        wmldck->maxstack+=10;
        ptr = wmldck->stack;
        wmldck->stack=Malloc(wmldck->maxstack);
	    ErrFatalDisplayIf (!wmldck->stack, "Malloc(stack-resize)");
        while (i>0)
           wmldck->stack[--i]=ptr[i];
        Free(ptr);
    }
    wmldck->stack[wmldck->stackDepth++] = wmldck->tag;

    return(0);    
}

void  freewmlstack(WMLDeckPtr wmldck)
{
    wmldck->stackDepth=0;
    if (wmldck->stack != NULL){
        Free(wmldck->stack);
        wmldck->stack = NULL;
    }
     
}

unsigned char popwmlstack(WMLDeckPtr wmldck)
{
    unsigned char c;

    if (wmldck->stack == NULL)
        return(-1);
    wmldck->stackDepth--; 
    c = wmldck->stack[wmldck->stackDepth];        
    if (wmldck->stackDepth == 0) {
        Free(wmldck->stack);
        wmldck->stack = NULL;
    }
    return(c);    
}

void renderLn(GlobalsType *g)
{
#ifndef __palmos__	
    fprintf(stdout,"\n");
#else
    palmprintln(g);
#endif __palmos__

}


void renderString(WMLDeckPtr wmldck, char *string, GlobalsType *g)
{
    if (wmldck->action == ACTION_RENDER)
#ifndef __palmos__	
        fprintf(stdout,string);
#else
        palmprintf(string,g);
#endif __palmos__

}

void renderImage(WMLDeckPtr wmldck, GlobalsType *g)
{
 char   					url[MAX_URL_LEN+1];
 Int16  					len;
 Int16  					sock;
 unsigned char				*imgBuf= NULL;
 UInt32 					maxTimer;
 WSPPDU       				wsppdu;
 char                       *wmlSrc;
 
//    FrmSetTitle(FrmGetActiveForm(),"Loading Image");
    wmlSrc= getAttributes(wmldck,WML_src,g);
	if(StrNCaselessCompare(wmlSrc,"MTCH:",5)==0)
		return;
    wapCreateUrlStr(wmlSrc,url,g);
    imgBuf=Malloc(DEF_RECEIVE_BUF+1);       
    ErrFatalDisplayIf (!imgBuf, "Malloc Failed");
    sock = wapGetUrl(&(g->conn), url,false,g);
    maxTimer = TimGetTicks() + (REPLY_TIMEOUT * SysTicksPerSecond());
	do { //TODO: this is no good, just a quick hack to get it working: an active loop
    	len = WspGetReply(sock,imgBuf,DEF_RECEIVE_BUF,maxTimer,g);
	}
	while(len==0);
    if (sock >= 0)
        close (sock);
	if (len > 0){ // looks like we've got at least a header
        if (WspDecode(imgBuf,len, &wsppdu ) == 0) {
#ifdef __palmos__	
			palmDrawImage(&wsppdu,g);
#endif __palmos__
			WspFreePdu(&wsppdu);
		}
	}
    if (imgBuf) {
        Free(imgBuf);
    }		
	   
}

void srcString(WMLDeckPtr wmldck, char *string, GlobalsType *g)
{
    if (wmldck->action == ACTION_SRC)
#ifndef __palmos__	
        fprintf(stdout,string);
#else
        palmsrcprintf(string,g);
#endif __palmos__

}

void srcIndent(WMLDeckPtr wmldck, GlobalsType *g)
{
    int i;

    if (wmldck->action == ACTION_SRC)
        for (i=0;i<wmldck->stackDepth;i++)
            srcString(wmldck," ",g);

}

void srcTag(WMLDeckPtr wmldck, GlobalsType *g)
{
    if ((wmldck->tag >=5) && (wmldck->tag <= 0x3F )) {
        srcString(wmldck,"<",g);  
        srcString(wmldck,wmltag[wmldck->tag-5],g);    
    }
}


void srcAttr(WMLDeckPtr wmldck, unsigned char attr_id, char *val,GlobalsType *g)
{
    if ((attr_id >=5) && (attr_id <= 0x5F )) {
        srcString(wmldck," ",g);  
        srcString(wmldck,attrStart[attr_id-05],g);    
        if (val[0]!=0) {
            srcString(wmldck,val,g);
            srcString(wmldck,"\"",g);
        }

    }
}



int checkCards(int startoffset, WMLDeckPtr wmldck, GlobalsType *g)
{
int off;
CardPtr tmp,idx;
Char *cptr;

    switch (wmldck->tag) {
    case WML_card :
	    off = wmldck->offset;
		tmp = Malloc(sizeof(CardType));
	    ErrFatalDisplayIf (!tmp, "Malloc Failed");
		tmp->offset = wmldck->offset;
		tmp->next = NULL;
        cptr = getAttributes(wmldck,WML_id,g);
		if (cptr){
			tmp->card_id = Malloc(StrLen(cptr)+1);
		    ErrFatalDisplayIf (!tmp->card_id, "Malloc2 Failed");
			StrCopy(tmp->card_id,cptr);
		}else
		{
			tmp->card_id = NULL;
		}
		if ( wmldck->card == NULL)
		{
			wmldck->card = tmp;
		}else
		{
		   idx = wmldck->card;
		   while(idx->next != NULL){
		   		idx = idx->next;
		   }
		   idx->next = tmp;
		}						
		wmldck->offset = off;
        getAttributes(wmldck,END,g);
        break;  
/*
    case WML_template : 
        wmldck->template = startoffset; 
        palmprintf("GOT Template:'");
        palmprintf(getAttributes(wmldck,WML_id));
		palmprintf("' ");
        break;
*/
	default:
        getAttributes(wmldck,END,g);
    }
    return(0);      
}


int getAttributeValue(WMLDeckPtr wmldck,char attrstr[])
{
    unsigned char c;
    int l,j;

            attrstr[0] = 0;
            j=0;
            c = wmldck->data[wmldck->offset++];
            while ((c > 128) || (c==ENTITY) || (c == STR_I) || (c == STR_T)   
                   || (c==SWITCH_PAGE) || (c==PI) 
                   || ((c >= EXT_I_0) && (c <= EXT_I_2))
                   || ((c >= EXT_T_0) && (c <= EXT_T_2))) {

                switch (c) {
                case ENTITY:
                    WspReadInt(wmldck->data,&(wmldck->offset)); 
                    DisplayError("Unhandled WML ENTITY");
					attrstr[j++]='<';
					attrstr[j++]='!';
					attrstr[j++]='>';
                    attrstr[j] = 0;
                    break;

                case STR_I:
                case EXT_I_0:
                case EXT_I_1:
                case EXT_I_2:     
                    while (wmldck->data[wmldck->offset] != 0) {
                        attrstr[j++]=wmldck->data[wmldck->offset++];
                    }
                    attrstr[j] = 0;
                    wmldck->offset++;   
                    break;

                case PI: // processing instruction
                    DisplayError("Unhandled WML PI");                
                    break; 

                case EXT_T_0:
                case EXT_T_1:
                case EXT_T_2: // 
                    l = WspReadInt(wmldck->data,&(wmldck->offset));
					attrstr[j++]=c;
                    while (wmldck->strTable[l] !=0)
                        attrstr[j++] = wmldck->strTable[l++];                   
					attrstr[j++]=END;
                    attrstr[j] = 0;
                    // !! TODO : handle escape/noescape
					if (c==EXT_T_0)
						;//escaped
					if (c==EXT_T_0)
						;//unescaped
                    break;
                case STR_T: 
                    l = WspReadInt(wmldck->data,&(wmldck->offset));
                    while (wmldck->strTable[l] !=0)
                        attrstr[j++] = wmldck->strTable[l++];                   
                    attrstr[j] = 0;
                    break;  
                case EXT_0:
                case EXT_1:
                case EXT_2:
                    //Single byte extension tokens		
                    l = wmldck->data[wmldck->offset++];
                    break;
                case OPAQUE: 
                    DisplayError("UNHANDELD OPAQUE");
                    break;

                default:
                    if ((c >=0x85) && (c <= 0xA1 )) {
                        l=0;
                        while (attrVal[c-0x85][l]!=0)
                            attrstr[j++] = attrVal[c-0x85][l++];                    
                        attrstr[j] = 0;
                    } else
                        DisplayError("UNKNOWN ATTRIBUTE"); 
                } //switch	
                c = wmldck->data[wmldck->offset++];
            } //while
        wmldck->offset--;
		return(0);
}

char *getAttributes(WMLDeckPtr wmldck, unsigned char attr, GlobalsType *g)
{
//    static Char attrstr[128];
//	char tmpstr[128];
    unsigned char c,attr_id;

	wmldck->attrstr[0]=0;
    if (wmldck->data[wmldck->offset] & WBXML_has_attributes) {
        wmldck->offset++;
        c= wmldck->data[wmldck->offset++];
        while (c != END) {
            attr_id = c;
			if (attr_id == attr){
   			   getAttributeValue(wmldck,wmldck->attrstr);
			   return(wmldck->attrstr);
			}else{
   			   getAttributeValue(wmldck,wmldck->tmpstr);
			}
            if (wmldck->action == ACTION_SRC)
                srcAttr(wmldck, attr_id,wmldck->tmpstr,g);
            c = wmldck->data[wmldck->offset++];
        }//while
        wmldck->offset--;
    }
    return(NULL);
}

int readElement(WMLDeckPtr wmldck, GlobalsType *g)
{
    int startoffset;
    unsigned char c;
	Char *attr;

    startoffset = wmldck->offset;
    wmldck->tag = wmldck->data[wmldck->offset] & WBXML_tag_id;
    c = wmldck->data[wmldck->offset];

    if ((wmldck->tag >=5) && (wmldck->tag <= 0x3F )) {
        switch (wmldck->action) {
        case ACTION_RENDER :
			switch(wmldck->tag) {
    		case WML_card :
        		attr = getAttributes(wmldck,WML_title,g);
				if (attr)
	        		FrmSetTitle(FrmGetActiveForm(),attr);
				else
	        		FrmSetTitle(FrmGetActiveForm(),"Wapuniverse");				
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,WML_ontimer,g);
				if (attr){
					if (wmldck->ontimerUrl)
						free(wmldck->ontimerUrl);
					wmldck->ontimerUrl = Malloc(StrLen(attr)+1);
					ErrFatalDisplayIf (!wmldck->ontimerUrl, "Malloc Failed");
					StrCopy(wmldck->ontimerUrl,attr);						
				}
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_timer:
				attr = getAttributes(wmldck,WML_value,g);
				if (attr){
						wmldck->timerVal=StrAToI(attr);
				}else{
                        DisplayError("Timer but no Value!"); 				
				}
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);			
			case WML_br:
			case WML_p:
                renderLn(g);
	            attr = getAttributes(wmldck,END,g);
				break;
			case WML_img:
				renderImage(wmldck,g);
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_u:
				underline(true);
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_b:
			case WML_strong:
				bold(true);
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_prev:
//				hyperLink(false,"","<PREV/>");
				break;
			case WML_do:
				attr = getAttributes(wmldck,WML_type_accept,g);
				if (attr){
					hyperLink(true,NULL,NULL,g);
				}
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);				
				break;				
			case WML_anchor:
				hyperLink(true,NULL,NULL,g);
				break;
			case WML_go:
				if ( (attr = getAttributes(wmldck,WML_href_http,g))){
			    	hyperLink(false,"http://",attr,g);
				}else {
					wmldck->offset=startoffset;
	 				if ( (attr = getAttributes(wmldck,WML_href,g))){
				    	hyperLink(false,"",attr,g);				
					}else{
						wmldck->offset=startoffset;
	 				    if ( (attr = getAttributes(wmldck,WML_href_https,g))){
				    	    hyperLink(false,"https://",attr,g);				
					    }
					}
				}
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_a:	
				if ( (attr = getAttributes(wmldck,WML_href_http,g))){
			    	hyperLink(true,"http://",attr,g);
				}else {
					wmldck->offset=startoffset;
	 				if ( (attr = getAttributes(wmldck,WML_href,g))){
				    	hyperLink(true,"",attr,g);				
					}else{
						wmldck->offset=startoffset;
	 				    if ( (attr = getAttributes(wmldck,WML_href_https,g))){
				    	    hyperLink(true,"https://",attr,g);				
					    }
					}
				}
				attr = getAttributes(wmldck,END,g);
				break;
			case WML_input:
				if ( (attr = getAttributes(wmldck,WML_name,g))){
	                wmlInput(attr,g);
				} 							
				wmldck->offset=startoffset;
				if ( (attr = getAttributes(wmldck,WML_value,g))){
	                wmlInputValue(attr,g);
				} 							
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);
			    break;
			case WML_select:
				if ( (attr = getAttributes(wmldck,WML_name,g))){
	                wmlSelect(attr,g);
				} 							
				wmldck->offset=startoffset;
				if ( (attr = getAttributes(wmldck,WML_ivalue,g))){
	                wmlSelectValue(attr,g);
				} 							
				wmldck->offset=startoffset;
				if ( (attr = getAttributes(wmldck,WML_value,g))){
	                wmlSelectValue(attr,g);
				} 							
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);			
				break;
			case WML_option:
				g->Render=false;
				if ( (attr = getAttributes(wmldck,WML_value,g))){
	                wmlSelectOption(attr,g);
				} 							
				wmldck->offset=startoffset;
				attr = getAttributes(wmldck,END,g);							
				break;
			default:
				attr = getAttributes(wmldck,END,g);
			}
            break;

        case ACTION_SRC :
            srcTag(wmldck,g);                   
            getAttributes(wmldck,END,g);
            break;
        case ACTION_PARSE :
            checkCards(startoffset,wmldck,g);
            break;
        default :
            getAttributes(wmldck,END,g);

        }       
    }
    // check for content
    if (c & WBXML_has_content) {
        pushwmlstack(wmldck);
        srcString(wmldck,">\n",g);
        srcIndent(wmldck,g);
    } else {
        srcString(wmldck,"/>\n",g);
        srcIndent(wmldck,g);
    }
    return(0);
}


int wbxml_parse_header(WSPPDUPtr wsppdu, WMLDeckPtr wmldck)
{
    int j;
    int strTableSz;

    wmldck->stackDepth = 0;
    wmldck->stack = NULL;
    wmldck->offset = 0;
    wmldck->data = wsppdu->data;
    wmldck->version = wmldck->data[wmldck->offset++];
    wmldck->publicID = wmldck->data[wmldck->offset++];
    if (wmldck->publicID==0)
        wmldck->offset++;
    // get the CharSet
    wmldck->charSet = wmldck->data[wmldck->offset++]; // not implemented yet ...
    // get the StringTable size, allocate memory and load it.
    strTableSz = WspReadInt(wmldck->data,&(wmldck->offset));
    wmldck->strTable=Malloc(strTableSz+1);
	wmldck->attrstr=Malloc(256);
	wmldck->tmpstr=Malloc(256);
    if ((!wmldck->strTable)||(!wmldck->attrstr) || (!wmldck->tmpstr) ) {
        ErrDisplay("Malloc");
        return(WML_ERR_MEM);
    }
    for (j=0;j<strTableSz;j++)
        wmldck->strTable[j] = wmldck->data[wmldck->offset++];
    wmldck->strTable[j]= 0;
    wmldck->decksize = wsppdu->length - wmldck->offset;
    wmldck->data = (wsppdu->data)+wmldck->offset;
    wmldck->offset = 0;
    return(0);
}


int wbxml_parse(WSPPDUPtr wsppdu, int action, WMLDeckPtr wmldck, int offset, GlobalsType *g)
{
	unsigned char *temp,c;
    int j;

    if (wsppdu->data==NULL) {
        return -1;
    }
	temp = Malloc (1024);
	ErrFatalDisplayIf (!temp, "Malloc Failed");
    wbxml_parse_header(wsppdu, wmldck);  
    wmldck->action = action;
	wmldck->offset = offset;
    // Parse the WBXML bytecode
    while (wmldck->offset < wmldck->decksize) {
        switch (wmldck->data[wmldck->offset]) {
        case SWITCH_PAGE :
            DisplayError("SWITCH_PAGE WMLC");
            break;
        case END:
            c= popwmlstack(wmldck);
            if (wmldck->action== ACTION_SRC) {
                srcString(wmldck,"\n",g);
                srcIndent(wmldck,g);
                srcString(wmldck,"</",g);
                srcString(wmldck,wmltag[c-5],g);
                srcString(wmldck,">",g);
            }
			if (wmldck->action== ACTION_RENDER) {
				switch(c) {
				case WML_option:
	                wmlSelectOptionTxt(temp,g);
					g->Render=true;
					break;
				case WML_select:
					wmlSelectEnd(g);
					break;
				case WML_u:
					underline(false);
					break;
				case WML_strong:
				case WML_b:
					bold(false);
					break;
				case WML_anchor:
				case WML_go:
				case WML_a:	
					hyperLink(false,NULL,NULL,g);
					break;
				}
			}
            if (wmldck->stackDepth == 0){  // end of <WML>, needed because angelOne seems to sent some trailing garbage.
                freewmlstack(wmldck);
				Free(temp);
                return(0);
            }
            break;
        case ENTITY:  // 0x02
//            DisplayError("ENTITY WMLC");
            wmldck->offset++;
            j = WspReadInt(wmldck->data,&(wmldck->offset)); // UCS-4 character code
			temp[0]=j; temp[1]=0;
			// TODO : should map UCS-4 to something printable ??						
            if (wmldck->action== ACTION_SRC)
                srcString(wmldck,temp,g);
            if ((wmldck->action== ACTION_RENDER)&& (g->Render==true))
                renderString(wmldck,temp,g);
            wmldck->offset--;
            break;
        case STR_I: //Inline String follows
            wmldck->offset++;
            j=0;
            while (wmldck->data[wmldck->offset] != 0) {
                temp[j++]=wmldck->data[wmldck->offset++];   
            }
            temp[j] = 0;
            if (wmldck->action== ACTION_SRC)
                srcString(wmldck,temp,g);
            if ((wmldck->action== ACTION_RENDER)&& (g->Render==true))
                renderString(wmldck,temp,g);
            break;
        case EXT_I_0:
        case EXT_I_1:
        case EXT_I_2:
        case EXT_T_0:
        case EXT_T_1:
        case EXT_T_2:
        case EXT_0:
        case EXT_1:
        case EXT_2:
        case OPAQUE:
            DisplayError("WML OPAQUE Extension in WMLC");
            break;
        case PI:
            DisplayError("PI not supported in WMLC");
            break;
        case STR_T: 
            wmldck->offset++;
            j = WspReadInt(wmldck->data,&(wmldck->offset));
			StrCopy(temp,wmldck->strTable+j);
            if (wmldck->action== ACTION_SRC)
                srcString(wmldck,temp,g);
            if ((wmldck->action== ACTION_RENDER) && (g->Render==true))
                renderString(wmldck,temp,g);
            wmldck->offset--;
            break;
        default:
            readElement(wmldck,g);
        }
        wmldck->offset++;
    }

    freewmlstack(wmldck);
	Free(temp);
    return(0);
}

int wbxml_free_wmldck(WMLDeckPtr wmldck)
{
CardPtr tmp;

    if (wmldck->attrstr != NULL)
        Free(wmldck->attrstr);
    wmldck->attrstr = NULL;
    if (wmldck->tmpstr != NULL)
        Free(wmldck->tmpstr);
    wmldck->tmpstr = NULL;
    if (wmldck->strTable != NULL)
        Free(wmldck->strTable);
    wmldck->strTable = NULL;
    if (wmldck->stack != NULL)
        Free(wmldck->stack);
    wmldck->stack = NULL;
	while (wmldck->card != NULL){
		tmp = wmldck->card;
		wmldck->card = tmp->next;
		if(tmp->card_id){
			Free(tmp->card_id);
		}
		Free(tmp);
	}
	

    return(0);
}


