//---------------------------------------------------------------------------
// wsp.c
// Contains all platform independant WAP/WSP code 
//
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
//
// $Workfile: wsp.c $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.6 $
//
// $Header: /cvsroot/wapuniverse/wapuniverse/src/wsp.c,v 1.6 2000/11/11 20:51:09 wapuniverse Exp $
//
//---------------------------------------------------------------------------

#include "wsp.h"
#include "WAPUniverse.h"

static char *wspStatusCodeStr[] = { 
	    "Multiple choices", // 0x30
	    "Moved permanently", // 0x31
        "Moved temporarily", // 0x32
        "See other", // 0x33
        "Not modified", // 0x34
        "Use proxy", // 0x35
        "-", // 0x36
        "-", // 0x37
        "-", // 0x38
        "-", // 0x39
        "-", // 0x3A
        "-", // 0x3B
        "-", // 0x3C
        "-", // 0x3D
        "-", // 0x3E
        "-", // 0x3F
        "Bad request", // 0x40
        "Unauthorized", // 0x41
        "Payment required", // 0x42
        "Forbidden", // 0x43
        "Not found", //0x44
        "Method not Allowed", //0x45
        "Not acceptable", //0x46
        "Proxy auth. required", //0x47
        "Request timeout", //0x48
        "Conflict", //0x49
        "Gone", //0x4A
        "Length required", //0x4B
        "Precondition failed", //0x4C
        "Request entity too large", //0x4D
        "Request-URI too large", // 0X4E
        "Unsupported media type", // 0x4F
        "-", // 0x50
        "-", // 0x51
        "-", // 0x52
        "-", // 0x53
        "-", // 0x54
        "-", // 0x55
        "-", // 0x56
        "-", // 0x37
        "-", // 0x58
        "-", // 0x59
        "-", // 0x5A
        "-", // 0x5B
        "-", // 0x5C
        "-", // 0x5D
        "-", // 0x5E
        "-", // 0x5F
        "internal server error", // 0x60
        "Not implemented", // 0x61
        "Bad Gateway", // 0x62
        "Service unavailable", // 0x63
        "Gateway Timeout", // 0x64
        "HTTP version not supported" // 0x65
};  

char *WspHostErrStr()
{
    static char msgstr[32];

    if (errno != 0)
        SysErrString(errno, msgstr, 32);
    else
        msgstr[0] = '\0';

    return(msgstr);
}


                                            

int WspReadInt(char *instr, int *i)
{
	int v = 0;
	unsigned char c;
	
	do {
		c = instr[(*i)++];
		v = (v << 7)|(c & 0x7f);
	}while (( c & 0x80) !=0);
	
	return (v);
	
}

int WspDecode (char *instr, int inlen, WSPPDUPtr wsppdu)
{
 int j,i;
 int sz;

  i=0;
  wsppdu->data = NULL;
  wsppdu->transaction_id = instr[i++];
  wsppdu->pdu_type = instr[i++];
  switch(wsppdu->pdu_type) {
	case  WSP_PDU_REPLY : // reply
  		wsppdu->status_code = instr[i++];
        if (wsppdu->status_code != WSP_STATUS_OK) {
            if ((wsppdu->status_code >= 0x30) && (wsppdu->status_code <= 0x65)) {
                MyErrorFunc("REPLY STATUS NOT OK:",wspStatusCodeStr[(wsppdu->status_code)-0x30]);
            }else{
                MyErrorFunc("REPLY STATUS NOT OK:","unkown");
            }
            return(-wsppdu->status_code);
        }
	        // get the content type and header size, skip for the moment
  		sz = WspReadInt(instr,&i);  
  		i+=sz;  //TODO: don't skip but do something with it
  		// Data
  		sz = inlen-i;
		wsppdu->data=Malloc(sz+1); 
   		ErrFatalDisplayIf (!wsppdu->data, "WspDecode: Malloc Failed");
  		for (j=0;j<(sz);j++)
    			wsppdu->data[j] = instr[i++];
  		wsppdu->length=sz;
		break;
	default:
			DisplayError("Unhandled PDUTYPE !!");
		break;
  }
  return(0);
}

int WspFreePdu(WSPPDUPtr wsppdu)
{
	if (wsppdu->data != NULL)
		Free(wsppdu->data);
	wsppdu->data = NULL;
    return(0);
}

Int16 WspGetReply(Int16 sock, char *buf, Int16 buflen, UInt32 maxTimer,GlobalsType *g)
{
    Int16     ret; 

    // please don't turn me off now
    EvtResetAutoOffTimer();     
    ret = recv(sock,buf,buflen,0);
    if ((errno==netErrTimeout) && ( TimGetTicks() < maxTimer ) ) {
        return(0);
    }
    if (ret <= 0) {
        g->state = BS_IDLE;
        FrmSetTitle(FrmGetActiveForm(),"WAPUniverse");
        if (ret == 0) {
            DisplayError("recv failed/Closed by remote");        
			ret = -1;
        }
        MyErrorFunc("getReply ",WspHostErrStr());
        if (errno == ENOMEM) {
            DisplayError("ENOMEM");        
        }
    } else {
        g->state = BS_WSP_DECODE;
    }
    // close socket if needed              
    if (sock >= 0)
        close (sock);
    sock = -1;
    return(ret);
}



