#ifndef _wsp_h_
#define _wsp_h_
//---------------------------------------------------------------------------
// wsp.h
// Contains all platform independant WAP/WSP code 
//
// Project: WAPUniverse for PalmOS
// Copyright  1999-2000 Filip Onkelinx
//
// http://www.wapuniverse.com/
// filip@onkelinx.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
//
//
// $Workfile: wsp.h $  
// $Author: wapuniverse $  
// $Date: 2000/11/11 20:51:09 $     
// $Revision: 1.6 $
//
// $Header: /cvsroot/wapuniverse/wapuniverse/src/wsp.h,v 1.6 2000/11/11 20:51:09 wapuniverse Exp $
//
//---------------------------------------------------------------------------
#include "WAP.h"
#include "WAPUniverse.h"

// WSP SDU size, according to WAP WSP 28-May-1999 V1.1 (SPEC-WSP-19990528.PDF Ch8.3.3)
#define DEF_RECEIVE_BUF 1400

#define WSP_ERR_BASE  -2000
#define WSP_ERR_MEM   -2001

// WSP PDU Types, according to WAP WSP 28-May-1999 V1.1 (SPEC-WSP-19990528.PDF Appendix A p87)
#define WSP_PDU_CONNECT 	    0x01
#define WSP_PDU_CONNECTREPLY 	0x02
#define WSP_PDU_REDIRECT 	    0x03
#define WSP_PDU_REPLY 		    0x04
#define WSP_PDU_DISCONNECT 	    0x05
#define WSP_PDU_PUSH 		    0x06
#define WSP_PDU_CONFIRMEDPUSH 	0x07
#define WSP_PDU_SUSPEND 	    0x08
#define WSP_PDU_RESUME 		    0x09
#define WSP_PDU_GET 		    0x40
#define WSP_PDU_OPTIONS		    0x41
#define WSP_PDU_HEAD		    0x42
#define WSP_PDU_DELETE		    0x43
#define WSP_PDU_TRACE		    0x44
#define WSP_PDU_POST		    0x60
#define WSP_PDU_PUT		        0x61

// WSP STATUS CODES, according to WAP WSP 28-May-1999 V1.1 (SPEC-WSP-19990528.PDF Appendix A p88)
#define WSP_STATUS_OK           0x20


extern int WspDecode(char *instr, int inlen, WSPPDUPtr wsppdu);
extern int WspFreePdu(WSPPDUPtr wsppdu);
extern int WspReadInt(char *instr, int *i);
extern Int16 WspGetReply(Int16 sock, char *buf, Int16 buflen, UInt32 maxTimer,GlobalsType *g);
extern char *WspHostErrStr();

#endif _wsp_h_
